package com.airfrance.squalecommon.daolayer.message;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.provider.persistence.hibernate.AbstractDAOImpl;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.enterpriselayer.businessobject.message.MessageBO;

/**
 * Couche DAO pour les messages
 */
public class MessageDAOImpl extends AbstractDAOImpl {
    /**
     * Instance singleton
     */
    private static MessageDAOImpl instance = null;

    /** initialisation du singleton */
    static {
        instance = new MessageDAOImpl();
    }

    /**
     * Constructeur prive
     * @throws JrafDaoException
     */
    private MessageDAOImpl() {
        initialize(MessageBO.class);
    }

    /**
     * Retourne un singleton du DAO
     * @return singleton du DAO
     */
    public static MessageDAOImpl getInstance() {
        return instance;
    }

    /** 
     * @param pSession la session
     * @param pKey la cl
     * @param pLang la langue (peut etre null si sans importance)
     * @return les messages dont la cl contient la chaine pass en paramtre
     * @throws JrafDaoException en cas d'chec
     */
    public Collection findWhereKey(ISession pSession, String pKey, String pLang) throws JrafDaoException {
        Collection coll = null;
        String whereClause = "";
        whereClause += "where " + getAlias() + ".id.key = 'news." + pKey+"'";
        if(pLang != null){
            whereClause += " AND " + getAlias()+ ".id.lang='" + pLang +"'";
        }
        coll = findWhere(pSession, whereClause);
        return coll;

    }

    /** 
     * @param pSession la session
     * @return toutes les langues disponibles
     * @throws JrafDaoException en cas d'chec
     */
    public Collection findLangs(ISession pSession) throws JrafDaoException {
        Collection coll = null;
        Collection result = new ArrayList(0);
        coll = findAll(pSession);
        if(coll != null){
            Iterator it = coll.iterator();
            while(it.hasNext()){
                MessageBO mess = (MessageBO)it.next();
                if(!result.contains(mess.getLang())){
                    result.add(mess.getLang());
                }
            }
        }
        return result;
    }

    /**
     * @param pSession la session
     * @param pMessBO le message  supprimer
     * @throws JrafDaoException en cas d'checs
     */
    public void removeMessage(ISession pSession, MessageBO pMessBO) throws JrafDaoException {
        String whereClause = "where " + getAlias() +".id.key like '%" + pMessBO.getKey() + "%'";
        removeWhere(pSession,whereClause);
    }
}
