//Source file: D:\\CC_VIEWS\\SQUALE_V0_0_ACT\\SQUALE\\SRC\\squaleCommon\\src\\com\\airfrance\\squalecommon\\enterpriselayer\\applicationcomponent\\ApplicationAdminApplicationComponentAccess.java

package com.airfrance.squalecommon.enterpriselayer.applicationcomponent.administration;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.commons.exception.JrafPersistenceException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.provider.accessdelegate.DefaultExecuteComponent;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.datatransfertobject.component.ApplicationConfDTO;
import com.airfrance.squalecommon.datatransfertobject.component.AuditDTO;
import com.airfrance.squalecommon.datatransfertobject.component.ComponentDTO;
import com.airfrance.squalecommon.datatransfertobject.component.ProjectConfDTO;
import com.airfrance.squalecommon.datatransfertobject.component.UserDTO;
import com.airfrance.squalecommon.enterpriselayer.applicationcomponent.ACMessages;
import com.airfrance.squalecommon.enterpriselayer.facade.component.ApplicationFacade;
import com.airfrance.squalecommon.enterpriselayer.facade.component.AuditFacade;
import com.airfrance.squalecommon.enterpriselayer.facade.component.ProjectFacade;

/**
 * <p>Title : ApplicationAdminApplicationComponentAccess.java</p>
 * <p>Description : Application component de configuration du projet </p>
 * <p>Copyright : Copyright (c) 2005</p>
 * <p>Company : AIRFRANCE</p>
 */
public class ApplicationAdminApplicationComponentAccess extends DefaultExecuteComponent {

    /**
     * log
     */
    private static final Log LOG = LogFactory.getLog(ApplicationAdminApplicationComponentAccess.class);

    /**
     * provider de persistence
     */
    private static final IPersistenceProvider PERSISTENTPROVIDER = PersistenceHelper.getPersistenceProvider();

    /**
     * Constructeur par dfaut
     * @roseuid 42CBFC010285
     */
    public ApplicationAdminApplicationComponentAccess() {
    }

    /**
     * Ajoute un projet  l'application courante
     * @param pProjectConf objet projet  ajouter
     * @param pApplicationConf objet application auquel on ajoute le projet
     * @return Integer : 0 pour la russite de la methode sinon 1 
     * @throws JrafEnterpriseException Exception JRAF
     * @deprecated utilisation de la methode saveProject(...)
     * @roseuid 42CBFC0102A3
     */
    public Integer addProject(ProjectConfDTO pProjectConf, ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        // Teste si le projet existe au sein de l'application
        Integer status = new Integer(0);
        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Ajout du projet  l'application
            pApplicationConf.getProjectConfList().add(pProjectConf);
            // Utilisation de ApplicationFacade pour modifier l'application
            ApplicationFacade.update(pApplicationConf, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { pProjectConf.getName()};
            String message = ACMessages.getString("ac.exception.applicationadmin.addproject", tab);
            status = new Integer(1);
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return status;
    }

    /**
     * sauve l'application en base de donnes
     * @param pApplicationConf ApplicationConfDTO associ
     * @return Integer : 0 pour la russite de la methode sinon 1 
     * @throws JrafEnterpriseException Exception JRAF
     */
    public Integer saveApplication(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        Integer status = new Integer(0);
        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Utilisation de ApplicationFacade pour modifier l'application
            ApplicationFacade.update(pApplicationConf, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(pApplicationConf.getId())};
            String message = ACMessages.getString("ac.exception.applicationadmin.saveapplication", tab);
            status = new Integer(1);
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return status;
    }
    
    /**
     * Modifie les paramtres de l'application concernant les dernires modifiations
     * @param pApplicationId l'id de l'application
     * @param pLastUser le nom du dernier utilisateur ayant modifi l'application
     * @param pLastUpdate la date de dernire modification
     * @throws JrafEnterpriseException si erreur
     */
    public void updateLastModifParams(Long pApplicationId, String pLastUser, Date pLastUpdate) throws JrafEnterpriseException {
        ApplicationFacade.updateLastModifParams(pApplicationId, pLastUser, pLastUpdate);
    }

    /**
     * Permet de savoir si une application existe en base grace a son nom et le cre en fonction du rsultat
     * @param pApplicationConf ApplicationConfDTO renseignant le nom de l'anne  vrifier
     * @param pUserCreating gestionnaire de l'application
     * @return Integer : 0 pour la russite de la methode sinon 1 
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFC0102FD
     */
    public ApplicationConfDTO createApplication(ApplicationConfDTO pApplicationConf, UserDTO pUserCreating) throws JrafEnterpriseException {

        // Initialisation
        ApplicationConfDTO projectConfDTO = null; // retour de la facade

        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Utilisation de ApplicationFacade pour insrer l'application
            projectConfDTO = ApplicationFacade.insert(pApplicationConf, pUserCreating, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(projectConfDTO.getId())};
            String message = ACMessages.getString("ac.exception.applicationadmin.createapplication", tab);
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return projectConfDTO;
    }

    /**
     * Permet de supprimer un projet d'une application
     * @param pProjectId l'ID du projet
     * @return le projet
     * @throws JrafEnterpriseException Exception JRAF
     */
    public ProjectConfDTO removeProject(Long pProjectId) throws JrafEnterpriseException {
        // Initialisation
        ISession session = null;
        ProjectConfDTO result = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Utilisation de ProjectFacade pour supprimer "logiquement" le projet
            result = ProjectFacade.delete(pProjectId, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(pProjectId)};
            String message = ACMessages.getString("ac.exception.applicationadmin.removeproject", tab);
            result = null;
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return result;
    }

    /**
     * Permet de dsactiver ou d'activer un projet d'une application
     * @param pProjectId l'ID du projet
     * @return le projet
     * @throws JrafEnterpriseException Exception JRAF
     */
    public ProjectConfDTO disactiveOrReactiveProject(Long pProjectId) throws JrafEnterpriseException {
        // Initialisation
        ISession session = null;
        ProjectConfDTO result = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Utilisation de ProjectFacade pour dsactiver le projet
            result = ProjectFacade.disactiveOrReactiveProject(pProjectId, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(pProjectId)};
            String message = ACMessages.getString("ac.exception.applicationadmin.removeproject", tab);
            result = null;
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return result;
    }

    /**
     * modifie un projet donn
     * @param pProjectConf projet  modifier
     * @param pApplicationConf application attache, si <code>null</code> modification des proprits uniquement
     * @return pProjectConfDTO si l'update s'est correctement deroul, sinon <code>null</code>
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFC01037F
     */
    public ProjectConfDTO saveProject(ProjectConfDTO pProjectConf, ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        ProjectConfDTO projectConf = null;
        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Utilisation de la ProjectFacade pour modifier le projet
            projectConf = ProjectFacade.update(pProjectConf, pApplicationConf, session);
            session.commitTransaction();
        } catch (Exception e) {
            String tab[] = { String.valueOf(pProjectConf.getId())};
            String message = ACMessages.getString("ac.exception.applicationadmin.saveproject", tab);
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return projectConf;
    }

    /**
     * Permet d'ajouter un milestone  une application existante
     * @param pAudit AuditDTO dfinissant un milestone
     * @return l'audit cre ou null si il y a eu un problme 
     * @throws JrafEnterpriseException Exception JRAF
     */
    public AuditDTO addMilestone(AuditDTO pAudit) throws JrafEnterpriseException {
        AuditDTO result = null;
        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            // Ajout du milestone au projet par AuditFacade
            result = AuditFacade.insertAudit(pAudit, session);
            session.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(pAudit.getApplicationId())};
            String message = ACMessages.getString("ac.exception.applicationadmin.addmilestone", tab);
            if (session != null) {
                session.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        }
        return result;
    }

    /**
     * Ajoute un audit de suivi  raliser sur l'application.
     * Les ventuels audits de suivi programms sont supprims au profit
     * du nouvel audit cr 
     * 
     * @param pAuditDTO audit de suivi  programmer
     * @return le DTO de l'audit insr (null en cas d'erreur d'insertion)
     * @throws JrafEnterpriseException Exception JRAF
     * @throws JrafPersistenceException exception Persistence
     */
    public AuditDTO addBranch(AuditDTO pAuditDTO) throws JrafEnterpriseException, JrafPersistenceException {
        ISession lSession = null;
        AuditDTO lAuditDTO = null;
        try {
            lSession = PERSISTENTPROVIDER.getSession();
            lSession.beginTransaction();
            // remplacement des audits de suivi programms
            lAuditDTO = AuditFacade.modifyNextNormalAudit(pAuditDTO.getApplicationId(), pAuditDTO, lSession);
            lSession.commitTransaction();
        } catch (Exception e) {
            // on rollback la transaction si ncessaire
            String tab[] = { String.valueOf(pAuditDTO.getApplicationId())};
            String message = ACMessages.getString("ac.exception.applicationadmin.addmilestone", tab);
            if (lSession != null) {
                lSession.rollbackTransaction();
            }
            LOG.fatal(message, e);
            throw new JrafEnterpriseException(message, e);
        } finally {
            lSession.closeSession();
        }
        return lAuditDTO;
    }

    /**
     * permet de rcuprer un ApplicationConfDTO  partir d'un identifiant
     * @param pApplicationConf ApplicationConfDTO renseignant l'ID de l'application
     * @return ApplicationConfDTO application concerne
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFC0103CF
     */
    public ApplicationConfDTO getApplicationConf(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        ApplicationConfDTO projectConf = new ApplicationConfDTO();
        projectConf = (ApplicationConfDTO) ApplicationFacade.getApplicationConf(pApplicationConf);
        return projectConf;
    }

    /**
     * Permet de reprogramm un audit sur une application au lendemain.
     * si il s'agit d'un audit de suivi, l'audit dj programm est supprim
     * @param pAudit AuditDTO dfinissant l'id de l'audit
     * @return l'audit reprogramm
     * @throws JrafEnterpriseException si erreur
     */
    public AuditDTO restartAudit(AuditDTO pAudit) throws JrafEnterpriseException {
        AuditDTO auditDTO = AuditFacade.restartAudit(pAudit);
        return auditDTO;
    }

    /**
     * Rcupre l'audit de jalon programm pour l'application d'id <code>pApplicationId</code> si il existe
     * @param pApplicationId l'id de l'application
     * @return l'audit trouv ou null sinon
     * @throws JrafEnterpriseException si erreur
     */
    public AuditDTO getMilestoneAudit(Long pApplicationId) throws JrafEnterpriseException {
        AuditDTO auditDTO = AuditFacade.getMilestoneAudit(pApplicationId);
        return auditDTO;
    }

    /**
     * Supprime l'audit de jalon d'id <code>removeAudit</code>
     * @param pAuditId l'id de l'audit
     * @return Integer : 0 pour la russite de la methode sinon 1 
     * @throws JrafEnterpriseException si erreur
     */
    public Integer removeAudit(Long pAuditId) throws JrafEnterpriseException {
        Integer status = AuditFacade.removeAudit(pAuditId);
        return status;
    }

    /**
     * Modifie l'audit de jalon d'id <code>removeAudit</code>
     * @param pAudit l'audit  mettre  jour
     * @return l'audit modifi
     * @throws JrafEnterpriseException si erreur
     */
    public AuditDTO modifyAudit(AuditDTO pAudit) throws JrafEnterpriseException {
        AuditDTO auditDto = AuditFacade.modifyAudit(pAudit);
        return auditDto;
    }

    /**
     * @param auditId l'id de l'audit (on veut rcuprer l'application correspondante)
     * @return l'application sous la forme d'un componentDTO
     */
    public ComponentDTO loadByAuditId(Long auditId) {
        ComponentDTO result = null;
        try {
            ISession session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            result = ApplicationFacade.loadByAuditId(auditId, session);
            session.commitTransaction();
        } catch (JrafDaoException e) {
            LOG.fatal(e, e);
        }
        return result;
    }

    /**
     * @return la liste des applications dfinies
     */
    public Collection listAll() {
        Collection result = new ArrayList(0);
        try {
            ISession session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            result = ApplicationFacade.listAll(session);
            session.commitTransaction();
        } catch (JrafDaoException e) {
            LOG.fatal(e, e);
        }
        return result;
    }

    /**
     * Ajoute un accs utilisateur  la date du jour
     * @param pApplications les applications de l'utilisateur
     * @param pMatricule le matricule de l'utilisateur
     * @param maxSize le nombre max d'accs  sauvegarder
     * @throws JrafEnterpriseException si erreur
     */
    public void addUserAccess(Collection pApplications, String pMatricule, Integer maxSize) throws JrafEnterpriseException {
        try {
            ISession session = PERSISTENTPROVIDER.getSession();
            session.beginTransaction();
            ApplicationFacade.addUserAccess(session, pApplications, pMatricule, maxSize);
            session.commitTransaction();
        } catch (JrafDaoException e) {
            LOG.fatal(e, e);
        }
    }
}
