//Source file: D:\\CC_VIEWS\\SQUALE_V0_0_ACT\\SQUALE\\SRC\\squaleCommon\\src\\com\\airfrance\\squalecommon\\enterpriselayer\\facade\\ApplicationFacade.java

package com.airfrance.squalecommon.enterpriselayer.facade.component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.provider.persistence.hibernate.facade.FacadeHelper;
import com.airfrance.jraf.spi.enterpriselayer.IFacade;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.component.ApplicationDAOImpl;
import com.airfrance.squalecommon.daolayer.component.AuditDAOImpl;
import com.airfrance.squalecommon.daolayer.profile.ProfileDAOImpl;
import com.airfrance.squalecommon.daolayer.profile.UserDAOImpl;
import com.airfrance.squalecommon.datatransfertobject.component.ApplicationConfDTO;
import com.airfrance.squalecommon.datatransfertobject.component.AuditDTO;
import com.airfrance.squalecommon.datatransfertobject.component.ComponentDTO;
import com.airfrance.squalecommon.datatransfertobject.component.UserDTO;
import com.airfrance.squalecommon.datatransfertobject.transform.component.ApplicationConfTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.component.ComponentTransform;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ApplicationBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.access.UserAccessBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.profile.ProfileBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.profile.UserBO;
import com.airfrance.squalecommon.enterpriselayer.facade.FacadeMessages;
import com.airfrance.squalecommon.util.messages.CommonMessages;

/**
 * Faade responsable de la gestion d'une application :<br /> 
 * - suppression, <br />
 * - tests d'existence, <br />
 * - rcupration, <br />
 * - insertion, mise  jour, ...<br />
 * d'une application.
 * @author ABOZ
 */
public class ApplicationFacade implements IFacade {

    /**
     * provider de persistence
     */
    private static final IPersistenceProvider PERSISTENTPROVIDER = PersistenceHelper.getPersistenceProvider();

    /** log */
    private static Log LOG = LogFactory.getLog(ApplicationFacade.class);

    /**
     * Dfinit l'heure  laquelle est programm le premier audit
     */
    public static final int HOUR_OF_AUDIT = 24;

    /**
     * Dfinit les minutes  laquelle est programm le premier audit
     */
    public static final int MINUTE_OF_AUDIT = 00;

    /**
     * Permet de rcuprer un objet ApplicationConfDTO  partir d'un objet presentant 
     * au moins l'ID de l'application.
     * @use by ApplicationAdministratorComponent
     * @param pApplicationConf ApplicationConfDTO contenant l'ID de l'application concerne
     * @return ApplicationConfDTO
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101B9
     */
    public static ApplicationConfDTO getApplicationConf(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {

        // Initialisation 
        ApplicationConfDTO applicationConfDTO = null; // retour de la methode
        Collection userBOs = null; // collection de UserBOs appartenant a l'application

        //Initialisation des variables temporaires
        ApplicationBO applicationBO = null;
        Long applicationID = new Long(pApplicationConf.getId());

        ISession session = null;

        try {

            session = PERSISTENTPROVIDER.getSession();

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            applicationBO = (ApplicationBO) applicationDAO.get(session, applicationID);

            UserDAOImpl userDAO = UserDAOImpl.getInstance();

            if (applicationBO != null) {

                userBOs = userDAO.findWhereApplication(session, applicationBO);
                applicationConfDTO = ApplicationConfTransform.bo2Dto(applicationBO, userBOs);

            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".getApplicationConf");
        } finally {
            FacadeHelper.closeSession(session, ApplicationFacade.class.getName() + ".getApplicationConf");
        }

        return applicationConfDTO;
    }

    /**
     * Permet de vrifier l'existence d'une application  partir de son nom
     * @use by ApplicationAdministratorComponent
     * @param pApplication ComponentDTO renseignant le nom d'une application
     * @return <code>true</code> si l'application existe, sinon <code>false</code>
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101C3
     */
    public static boolean exists(ComponentDTO pApplication) throws JrafEnterpriseException {

        // Nombre d'applications portant le meme nom
        int numberApplication = 0;
        boolean existsApplication = false;

        ISession session = null;

        try {

            session = PERSISTENTPROVIDER.getSession();

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            numberApplication = applicationDAO.countWhereName(session, pApplication.getName());

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".exists");
        } finally {
            FacadeHelper.closeSession(session, ApplicationFacade.class.getName() + ".exists");
        }

        if (numberApplication > 0) {
            existsApplication = true;
        } else {
            existsApplication = false;
        }

        return existsApplication;
    }

    /**
     * - Transforme les DTO en objets mtiers via des transformeurs.
     * @dev-squale Demande  la partie DAO de sauvegarder / mettre  jour les objets 
     * mtiers : <br />
     * - Un ApplicationBO<br />
     * - Des ProjectBO<br />
     * - Des UserBO<br />
     * - 0 ou 1 AuditBO<br />
     * - Des RightsBO<br />
     * @use by ApplicationAdministratorComponent
     * @param pApplicationConf configuration de l'application courante
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101C5
     */
    public static void update(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        update(pApplicationConf, null);
    }

    /**
     * Transforme les DTO en objets mtiers via des transformeurs.
     * Demande  la partie DAO de sauvegarder / mettre  jour les objets 
     * mtiers :<br />
     * - Un ApplicationBO<br />
     * - Des ProjectBO<br />
     * - Des UserBO<br />
     * - 0 ou 1 AuditBO<br />
     * - Des RightsBO<br />
     * @param pApplicationConf configuration de l'application courante
     * @param pSession session JRAF
     * @throws JrafEnterpriseException exception JRAF
     */
    public static void update(ApplicationConfDTO pApplicationConf, ISession pSession) throws JrafEnterpriseException {
        // Initialisation 
        ApplicationBO applicationBO = null; // retour du load pour l'application
        UserBO userBO = null; // retour du load pour l'utilisateur
        Long applicationID = new Long(pApplicationConf.getId());
        // identifiant de l'application
        ProfileBO profileBO = null; // profil des utilisateurs a ajouter

        try {
            // Cration d'une session locale si besoin
            if (pSession == null) {
                pSession = PERSISTENTPROVIDER.getSession();
            }
            // Initialisation des DAO
            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();
            UserDAOImpl userDAO = UserDAOImpl.getInstance();
            ProfileDAOImpl profileDAO = ProfileDAOImpl.getInstance();

            // On rcupre l'application en base
            applicationBO = (ApplicationBO) applicationDAO.get(pSession, applicationID);

            // Si celle-ci existe (code dfensif)
            if (applicationBO != null) {
                // Variables temporaires de sauvegarde des donnes en base avant transformation
                int previousFrequency = applicationBO.getAuditFrequency();
                String user = applicationBO.getLastUser();
                Date date = applicationBO.getLastUpdate();
                // On va sauvegarder la transformation seulement si il y a eu des modifications
                boolean toUpdate = applicationToUpdate(pApplicationConf, applicationBO);
                // Transformation du DTO en BO
                ApplicationConfTransform.dto2Bo(pApplicationConf, applicationBO);
                // On garde les anciennes valeurs qui seront updates seulement si on sauvegarde
                // l'application
                applicationBO.setLastUpdate(date);
                applicationBO.setLastUser(user);

                // Modification du type d'audit si besoin
                // Seulement dans le cas d'une application dont la frquence implique
                // le changement du type d'Audit (de jalon ou de suivi)
                if (previousFrequency != applicationBO.getAuditFrequency()) {
                    updateAudit(pSession, applicationBO, previousFrequency);
                }
                // On modifie les droits sur l'application
                toUpdate |= updateApplicationRights(pSession, pApplicationConf, applicationBO);
                // On sauvegarde l'application si la conf ou les droits ont chang
                if (toUpdate) {
                    // On modifie l'application, il faut mettre  jour les informations
                    // sur les dernires modifications
                    applicationBO.setLastUpdate(pApplicationConf.getLastUpdate());
                    applicationBO.setLastUser(pApplicationConf.getLastUser());
                    applicationDAO.save(pSession, applicationBO);
                }
            }
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".update");
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".update");
        }
    }

    /**
     * @param pDTO l'application  comparer
     * @param pBO l'application en base
     * @return true si l'objet en base  chang
     */
    private static boolean applicationToUpdate(ApplicationConfDTO pDTO, ApplicationBO pBO) {
        boolean toUpdate = !pDTO.getName().equals(pBO.getName());
        if (null != pBO.getServeurBO()) {
            toUpdate |= pDTO.getServeurDTO().getServeurId() != pBO.getServeurBO().getServeurId();
        }
        toUpdate |= pDTO.getPublic() != pBO.getPublic();
        toUpdate |= pDTO.getAuditFrequency() != pBO.getAuditFrequency();
        toUpdate |= pDTO.getResultsStorageOptions() != pBO.getResultsStorageOptions();
        toUpdate |= pDTO.getInProduction() != pBO.getInProduction();
        toUpdate |= pDTO.getExternalDev() != pBO.getExternalDev();
        return toUpdate;
    }

    /**
     * @param pSession la session hibernate
     * @param pApplicationConf l'application modifie
     * @param pAppliBO l'application  modifier
     * @return true si les modifications doivent tre sauvegardes
     * @throws JrafDaoException si erreur
     */
    private static boolean updateApplicationRights(ISession pSession, ApplicationConfDTO pApplicationConf, ApplicationBO pAppliBO) throws JrafDaoException {
        // Initialisation
        UserBO userBO = null; // retour du load pour l'utilisateur
        ProfileBO profileBO = null; // profil des utilisateurs a ajouter
        // Les DAO
        UserDAOImpl userDAO = UserDAOImpl.getInstance();
        ProfileDAOImpl profileDAO = ProfileDAOImpl.getInstance();
        // Retour de la mthode
        boolean usersChanged = false;

        // On itre sur les nouveaux profils
        Iterator usersIterator = pApplicationConf.getUsers().keySet().iterator();
        while (usersIterator.hasNext()) {
            String userName = (String) usersIterator.next();
            // rcupration du userBO associ au matricule
            userBO = (UserBO) userDAO.loadWithMatricule(pSession, userName);
            // Initialisation du profil de lecteur par defaut  
            profileBO = profileDAO.loadByKey(pSession, (String) pApplicationConf.getUsers().get(userName));
            // Si l'utilisateur n'existe pas, on le cre en base
            // Sinon on change la map du userBO et on sauvegarde en base
            if (userBO == null) {
                // Initialisation du matricule et du profil par defaut
                userBO = new UserBO();
                userBO.setDefaultProfile(profileBO);
                userBO.setMatricule(userName);
                // Attribution sur les droits
                Map applicationsRights = new HashMap();
                applicationsRights.put(pAppliBO, profileBO);
                userBO.setRights(applicationsRights);
                // On cre l'objet en base
                userDAO.create(pSession, userBO);
                usersChanged |= true;
            } else {
                // Remplacement ou affectation d'un nouveau profil sur
                // l'application concerne
                ProfileBO curProfil = (ProfileBO) userBO.getRights().get(pAppliBO);
                if (curProfil == null || !curProfil.equals(profileBO)) {
                    userBO.getRights().put(pAppliBO, profileBO);
                    userDAO.save(pSession, userBO);
                    usersChanged = true;
                }
            }
        }
        // Purge des utilisateurs qui ne doivent plus tre dfinis sur 
        // l'application
        usersChanged |= purgeUsers(pSession, pAppliBO, pApplicationConf.getUsers().keySet());
        return usersChanged;
    }

    /**
     * Purge des utilisateurs
     * @param pSession session
     * @param pApplicationBO application
     * @param pUserDefined utilisateurs dfinis
     * @return true si des droits ont t supprims
     * @throws JrafDaoException si erreur
     */
    static private boolean purgeUsers(ISession pSession, ApplicationBO pApplicationBO, Collection pUserDefined) throws JrafDaoException {
        boolean deleted = false;
        UserDAOImpl userDAO = UserDAOImpl.getInstance();
        Collection userBOs = userDAO.findWhereApplication(pSession, pApplicationBO);
        Iterator existingUsers = userBOs.iterator();
        while (existingUsers.hasNext()) {
            UserBO userBO = (UserBO) existingUsers.next();
            if (!pUserDefined.contains(userBO.getMatricule())) {
                userBO.getRights().remove(pApplicationBO);
                userDAO.save(pSession, userBO);
                deleted = true;
            }
        }
        return deleted;
    }

    /**
     * Mise  jour des audits sur une application
     * Lorsqu'une application est modifie, les audits qui y sont associs peuvent passer
     * d'un audit de type normal  un audit de type jalon. Dans ce cas, il faut supprimer
     * l'audit normal s'il en existe un. Dans le cas inverse, il faut crer un audit normal
     * et le rattacher  l'application.
     * @param pSession session
     * @param pApplicationBO application
     * @param pPreviousFrequency l'ancienne frquence
     * @throws JrafEnterpriseException si erreur
     * @throws JrafDaoException si erreur
     */
    private static void updateAudit(ISession pSession, ApplicationBO pApplicationBO, int pPreviousFrequency) throws JrafEnterpriseException, JrafDaoException {
        AuditDAOImpl auditDAO = AuditDAOImpl.getInstance();
        // Modification du type d'audit
        if (pPreviousFrequency == -1 && pApplicationBO.getStatus() == ApplicationBO.VALIDATED) {
            // Cration d'un audit de suivi seulement si l'application est valide
            AuditBO newAudit = createNewAudit();
            auditDAO.create(pSession, newAudit);
            pApplicationBO.addAudit(newAudit);
        } else {
            // Suppression ou modification des audits de suivi
            ComponentDTO applicationDTO = new ComponentDTO();
            applicationDTO.setID(pApplicationBO.getId());
            List audits = AuditFacade.getAudits(applicationDTO, null, null, AuditBO.NORMAL, AuditBO.NOT_ATTEMPTED);
            Iterator iterator = audits.iterator();
            while (iterator.hasNext()) {
                AuditDTO auditDTO = (AuditDTO) iterator.next();
                if (pApplicationBO.getAuditFrequency() == -1) {
                    // Le type des audits a chang, on supprime
                    AuditFacade.delete(auditDTO, pSession);
                } else { // La frquence a chang sans changer le type des audits
                    // On ajoute la diffrence entre les frquences
                    Date auditDate = auditDTO.getDate();
                    Calendar newDate = GregorianCalendar.getInstance();
                    newDate.setTime(auditDate);
                    newDate.add(Calendar.DATE, pApplicationBO.getAuditFrequency() - pPreviousFrequency);
                    auditDTO.setDate(newDate.getTime());
                    // On update l'audit
                    AuditFacade.modifyAudit(auditDTO);
                }
            }
        }
    }

    /**
     * Permet de crer en base une application (le premier audit sera cr  la validation) 
     * et vrifier si une application portant le mme nom n'existe pas dj
     * @dev-squale Ajouter l'utilisateur crant au applicationConfDTO
     * @use by ApplicationAdministratorComponent
     * @param pApplicationConf application existante
     * @param pUserCreating utilisateur qui cre l'application
     * @return ApplicationConfDTO avec le nom et l'ID de l'application en cours de cration
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101D9
     */
    public static ApplicationConfDTO insert(ApplicationConfDTO pApplicationConf, UserDTO pUserCreating) throws JrafEnterpriseException {
        return insert(pApplicationConf, pUserCreating, null);
    }

    /**
     * Permet de crer en base une application (le premier audit sera cr  la validation) 
     * et vrifier si une application portant le mme nom n'existe pas dj
     * @dev-squale Ajouter l'utilisateur crant au applicationConfDTO
     * @use by ApplicationAdministratorComponent
     * @param pApplicationConf application existant
     * @param pSession session JRAF
     * @param pUserCreating utilisateur gestionnaire 
     * @return ApplicationConfDTO avec le nom et l'ID de l'application en cours de cration
     * @except une exception si application dj existante ou  nom d'application invalide
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101E1
     */
    public static ApplicationConfDTO insert(ApplicationConfDTO pApplicationConf, UserDTO pUserCreating, ISession pSession) throws JrafEnterpriseException {

        //Initialisation 
        Long userID = new Long(pUserCreating.getID());
        // identifiant de l'utilisateur
        UserBO userBO = null; // retour de UserDAO
        ProfileBO profileBO = null; // retour de ProfileDAO

        // Initialisation des variables temporaires
        ApplicationBO applicationBO = null;

        try {

            if (pSession == null) {
                pSession = PERSISTENTPROVIDER.getSession();
            }

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();
            UserDAOImpl userDAO = UserDAOImpl.getInstance();
            ProfileDAOImpl profileDAO = ProfileDAOImpl.getInstance();

            // Transformation du DTO en BO + creation de l'objet en base
            applicationBO = ApplicationConfTransform.dto2Bo(pApplicationConf);
            applicationBO = applicationDAO.create(pSession, applicationBO);

            userBO = (UserBO) userDAO.get(pSession, userID);

            /* Chargement du UserBO et ProfileBO correspondant au gestionnaire d'application
             * si une application du meme nom n'existe pas */
            if (applicationBO != null && userBO != null) {

                profileBO = (ProfileBO) profileDAO.loadByKey(pSession, ProfileBO.MANAGER_PROFILE_NAME);

                // Ajout d'une application dans la Map rights et update du UserBO charg
                userBO.getRights().put(applicationBO, profileBO);
                userDAO.save(pSession, userBO);

                Collection userBOs = new ArrayList();
                userBOs.add(userBO);

                pApplicationConf = ApplicationConfTransform.bo2Dto(applicationBO, userBOs);
            } else {
                pApplicationConf = null;
            }

        } catch (JrafDaoException e) {
            pApplicationConf = null;
            if (applicationBO != null) {
                LOG.info(FacadeMessages.getString("facade.exception.applicationfacade.insert.existence"), e);
            }
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".insert");
        }
        return pApplicationConf;
    }

    /**
     * permet de rcuprer l'objet ApplicationDTO par un ID
     * @use by ResultsComponent
     * @param pApplication ApplicationDTO renseignant l'id de l'application concerne
     * @return ApplicationDTO
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101EC
     */
    public static ComponentDTO getApplication(ComponentDTO pApplication) throws JrafEnterpriseException {

        // Initialisation du retour
        ComponentDTO applicationDTO = null;

        //Initialisation des variables temporaires
        ApplicationBO applicationBO = null;
        Long applicationID = new Long(pApplication.getID());

        ISession session = null;

        try {

            session = PERSISTENTPROVIDER.getSession();

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            applicationBO = (ApplicationBO) applicationDAO.get(session, applicationID);

            if (applicationBO != null) {
                applicationDTO = ComponentTransform.bo2Dto(applicationBO);
            } else {
                LOG.error(FacadeMessages.getString("facade.exception.applicationfacade.getApplication.applicationnull"));
            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".getApplication");
        } finally {
            FacadeHelper.closeSession(session, ApplicationFacade.class.getName() + ".getApplication");
        }

        return applicationDTO;
    }

    /**
     * Permet d'effacer une liste d'applications
     * @param pApplicationDTOs liste des ApplicationDTOs renseignant l'identifiant qui sont  supprimer
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101F4
     */
    public static void deleteApplicationConfList(Collection pApplicationDTOs) throws JrafEnterpriseException {
        deleteApplicationConfList(pApplicationDTOs, null);
    }

    /**
     * Permet d'effacer une liste d'applications
     * @param pApplicationDTOs liste de ApplicationDTOs renseignant l'identifiant qui sont  supprimer
     * @param pSession session JRAF
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB101F6
     */
    public static void deleteApplicationConfList(Collection pApplicationDTOs, ISession pSession) throws JrafEnterpriseException {

        try {
            if (pSession == null) {
                // si aucune session, on en rcupre une
                pSession = PERSISTENTPROVIDER.getSession();
            }
            // Rcupration d'une instance de ApplicationDAOImpl
            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            Iterator applicationIterator = pApplicationDTOs.iterator();
            ApplicationBO applicationTemp = null;

            // Suppression de la liste de ProjectConfBO
            while (applicationIterator.hasNext()) {
                Long applicationId = new Long(((ApplicationConfDTO) applicationIterator.next()).getId());
                // rcupration de l'application p/r  l'id
                applicationTemp = (ApplicationBO) applicationDAO.get(pSession, applicationId);
                // suppression de l'application
                applicationDAO.remove(pSession, applicationTemp);
            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".deleteApplicationConfList");
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".deleteApplicationConfList");
        }
    }

    /**
     * Permet de valider une liste de ApplicationConf
     * @use by ValidationComponent
     * @param pApplicationIDs liste des ids des applications  valider
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10209
     */
    public static void validateApplicationConfList(Collection pApplicationIDs) throws JrafEnterpriseException {
        validateApplicationConfList(pApplicationIDs, null);
    }

    /**
     * Permet de valider une liste de ApplicationConf
     * @use by ValidationComponent
     * @param pApplicationConfDTOs liste des des applications  valider
     * @param pSession session JRAF
     * @return l'ensemble des applications qui n'ont pas pu tre valides
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10212
     */
    public static List validateApplicationConfList(Collection pApplicationConfDTOs, ISession pSession) throws JrafEnterpriseException {

        // Initialisation 
        AuditBO newAudit = null; // Nouvel audit a ajouter
        List notValidated = new ArrayList();

        try {
            if (pSession == null) {
                pSession = PERSISTENTPROVIDER.getSession();
            }

            // Initialisation des DAO
            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();
            AuditDAOImpl auditDAO = AuditDAOImpl.getInstance();

            Iterator iterator = pApplicationConfDTOs.iterator();
            Long applicationIDTemp = null;
            ApplicationBO applicationToValidate = null;

            while (iterator.hasNext()) {

                applicationIDTemp = new Long(((ApplicationConfDTO) iterator.next()).getId());

                //Recuperation de applicationBO
                applicationToValidate = (ApplicationBO) applicationDAO.get(pSession, applicationIDTemp);

                // On vrifie qu'il n'y a pas d'application du mme nom en base
                // (possible lorsque les temps de rponse sont trop longs lorsqu'on cre une application)
                int existantApplication = applicationDAO.countWhereName(pSession, applicationToValidate);
                if (existantApplication > 1) {
                    // On ajoute son nom  l'ensemble des applications qui n'ont pas pu tre
                    // valide car une autre du mme nom existe en base
                    notValidated.add(applicationToValidate.getName());
                } else {

                    // Creation d'un nouvel audit pour le lendemain minuit si un audit de suivi est souhait
                    // et si aucun audit de suivi n'a dj t programm (cas d'un audit de test d'une application non valide)
                    List lList = auditDAO.findWhereComponent(pSession, applicationIDTemp.longValue(), null, null, AuditBO.NORMAL, AuditBO.NOT_ATTEMPTED);
                    if (applicationToValidate.getAuditFrequency() > 0 && lList.isEmpty()) {
                        newAudit = createNewAudit();
                        auditDAO.create(pSession, newAudit);
                    }

                    // Changement du status en validated et Ajout de l'audit nouvellement cr  l'application
                    applicationToValidate.setStatus(ApplicationBO.VALIDATED);
                    if (newAudit != null) {
                        applicationToValidate.addAudit(newAudit);
                    }

                    // Sauvegarde de l'objet
                    applicationDAO.save(pSession, applicationToValidate);
                }
            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".validateApplicationConfList");
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".validateApplicationConfList");
        }
        return notValidated;
    }

    /**
     * Permet de retourner une collection de ApplicationConfDTO  valider par 
     * l'administrateur du portail
     * @return Collection de ApplicationConfDTO applications  valider
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10215
     */
    public static Collection getApplicationConfList() throws JrafEnterpriseException {

        // Initialisation des variables
        Collection applicationBOs = null;
        Collection applicationConfDTOs = new ArrayList();
        Collection userBOs = null; // collection de UserBOs appartenant a l'application

        ISession session = null;

        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            // Recuperation des applications non valides
            applicationBOs = applicationDAO.findWhereStatus(session, ApplicationBO.IN_CREATION);

            Iterator iterator = applicationBOs.iterator();
            ApplicationBO applicationTemp = null;

            // Transformation des BOs en DTOs
            while (iterator.hasNext()) {
                // Pour chaque application
                applicationTemp = (ApplicationBO) iterator.next();

                if (applicationTemp != null) {
                    // rcupration de la liste d'utilisateur de cette application
                    userBOs = UserDAOImpl.getInstance().findWhereApplication(session, applicationTemp);
                    // Cration d'un ApplicationDTO  partir du ApplicationBO et de la liste de UserBO
                    applicationConfDTOs.add(ApplicationConfTransform.bo2Dto(applicationTemp, userBOs));
                }
            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".getApplicationConfList");
        } finally {
            FacadeHelper.closeSession(session, ApplicationFacade.class.getName() + ".getApplicationConfList");
        }

        return applicationConfDTOs;
    }

    /**
     * Permet de supprimer une application et toutes ses relations (audits et rsultats)
     * @dev-squale s'il n'y a pas de jalon existant, insrer le dernier audit dans le rfrentiel.
     * @use by PurgeComponent.purgeApplication()
     * @param pApplicationConf ApplicationConfDTO renseignant l'ID de l'application
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB1021E
     */
    public static void deleteAll(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        deleteAll(pApplicationConf, null);
    }

    /**
     * Permet de vrifier si une application possde un milestone.
     * @use by PurgeComponent.purgeApplication()
     * @param pApplicationConf ApplicationConfDTO renseignant l'ID de l'application
     * @return <code>true</code> si l'application possde un milestone sinon <code>false</code>
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10220
     */
    public static boolean existsMilestone(ApplicationConfDTO pApplicationConf) throws JrafEnterpriseException {
        return existsMilestone(pApplicationConf, null);
    }

    /**
     * Permet de supprimer une application et toutes ses relations (audits et rsultats)
     * @dev-squale s'il n'y a pas de jalon existant, insrer le dernier audit dans le rfrentiel.
     * @use by PurgeComponent.purgeApplication()
     * @param pApplicationConf ApplicationConfDTO renseignant l'ID de l'application
     * @param pSession session Hibernate
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10227
     */
    public static void deleteAll(ApplicationConfDTO pApplicationConf, ISession pSession) throws JrafEnterpriseException {

        // Initialisation
        Long applicationID = new Long(pApplicationConf.getId()); // identifiant application
        ApplicationBO applicationBO = null; // application a supprimer

        try {

            if (pSession == null) {
                pSession = PERSISTENTPROVIDER.getSession();
            }

            ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();

            // Chargement de l'application associe  l'identifiant application
            // Suppression du ApplicationBO associ et toutes ses relations 
            applicationBO = (ApplicationBO) applicationDAO.get(pSession, applicationID);
            applicationDAO.remove(pSession, applicationBO);

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".deleteAll");
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".deleteAll");
        }

    }

    /**
     * Permet de vrifier si une application possde un milestone.
     * @use by PurgeComponent.purgeApplication()
     * @param pApplicationConf ApplicationConfDTO renseignant l'ID d'une application
     * @param pSession session Hibernate
     * @return <code>true</code> si l'application possde un milestone sinon <code>false</code>
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB10230
     */
    public static boolean existsMilestone(ApplicationConfDTO pApplicationConf, ISession pSession) throws JrafEnterpriseException {

        //Initialisation du retour et objets temporaires
        boolean existsMilestone = true; // retour
        ApplicationBO applicationBO = null; // objet metier utilise
        int nbMilestones = -1; // nombre de milestone retourne par la DAO

        try {

            if (pSession == null) {
                // si aucune session n'est passe
                pSession = PERSISTENTPROVIDER.getSession();
            }
            // transformation du ApplicationConfDTO en ApplicationBO
            applicationBO = ApplicationConfTransform.dto2Bo(pApplicationConf);

            AuditDAOImpl auditDAO = AuditDAOImpl.getInstance();

            // Rcupration du nombre de milestone pour l'application
            nbMilestones = auditDAO.countWhereType(pSession, applicationBO, AuditBO.MILESTONE, AuditBO.TERMINATED);

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".existsMilestone");
        } finally {
            FacadeHelper.closeSession(pSession, ApplicationFacade.class.getName() + ".existsMilestone");
        }

        // si des milestone ont t trouv, on retourne true, on retourne false sinon
        if (nbMilestones > 0) {
            existsMilestone = true;
        } else {
            existsMilestone = false;
        }

        return existsMilestone;

    }

    /**
     * Permet d'ajouter un prochain audit  une application donn aprs validation de l'application
     * par un administrateur
     * @return AuditBO un nouvel audit cr pour le jour meme a minuit
     * @throws JrafEnterpriseException exception JRAF
     */
    private static AuditBO createNewAudit() throws JrafEnterpriseException {

        // Initialisation 
        AuditBO auditBO = new AuditBO(); // audit  ajouter

        // Date du premier audit initialis au jour meme  0:00
        Calendar myDate = GregorianCalendar.getInstance();
        myDate.add(GregorianCalendar.HOUR_OF_DAY, HOUR_OF_AUDIT - myDate.get(GregorianCalendar.HOUR_OF_DAY));
        myDate.add(GregorianCalendar.MINUTE, MINUTE_OF_AUDIT - myDate.get(GregorianCalendar.MINUTE));
        auditBO.setDate(myDate.getTime());

        // Ajout du commentaire de premier audit
        auditBO.setComments(CommonMessages.getString("bo.audit.comments.firstaudit"));

        auditBO.setStatus(AuditBO.NOT_ATTEMPTED);

        // name = date en string
        auditBO.setName(myDate.getTime().toString());

        // type dans le fichier de configuration
        auditBO.setType(AuditBO.NORMAL);

        return auditBO;
    }

    /**
     * Constructeur vide
     * @roseuid 42CBFFB10233
     */
    private ApplicationFacade() {
    }

    /**
     * @param auditId l'id de l'audit (on veut rcuprer l'application correspondante)
     * @param pSession la session
     * @return l'application sous la forme d'un componentDTO
     * @throws JrafDaoException en cas d'checs
     */
    public static ComponentDTO loadByAuditId(Long auditId, ISession pSession) throws JrafDaoException {
        ApplicationDAOImpl dao = ApplicationDAOImpl.getInstance();
        return ComponentTransform.bo2Dto(dao.loadByAuditId(pSession, auditId));
    }

    /** 
     * retourne la liste des applications dfinies
     * @param pSession la session
     * @return la liste des application
     * @throws JrafDaoException en cas d'checs
     */
    public static Collection listAll(ISession pSession) throws JrafDaoException {
        ApplicationDAOImpl dao = ApplicationDAOImpl.getInstance();
        Collection collDao = dao.findNotDeleted(pSession);
        Collection collDto = new ArrayList(0);
        Iterator it = collDao.iterator();
        while (it.hasNext()) {
            collDto.add(ComponentTransform.bo2Dto((ApplicationBO) it.next()));
        }
        return collDto;
    }

    /**
     * @param pSession la session
     * @param pApplications les applications de l'utilisateur
     * @param pMatricule le matricule de l'utilisateur
     * @param maxSize le nombre max d'accs  sauvegarder
     * @throws JrafDaoException si erreur
     */
    public static void addUserAccess(ISession pSession, Collection pApplications, String pMatricule, Integer maxSize) throws JrafDaoException {
        // Initialisation du DAO
        ApplicationDAOImpl appliDAO = ApplicationDAOImpl.getInstance();
        ApplicationBO appliBO = null;
        for (Iterator it = pApplications.iterator(); it.hasNext();) {
            Long appliId = new Long(((ComponentDTO) it.next()).getID());
            // On rcupre l'application
            appliBO = (ApplicationBO) appliDAO.get(pSession, appliId);
            if (null != appliBO && appliBO.getStatus() == ApplicationBO.VALIDATED) { // code dfensif
                // On ajoute l'accs utilisateur
                UserAccessBO userAccess = new UserAccessBO(pMatricule, Calendar.getInstance().getTime());
                appliBO.addUserAccess(userAccess, maxSize.intValue());
                // On sauvegarde l'application
                appliDAO.save(pSession, appliBO);
            }
        }
    }

    /**
     * @param pApplicationId l'id de l'application
     * @param pLastUser le nom du dernier utilisateur ayant modifi l'application
     * @param pLastUpdate la date de dernire modification
     * @throws JrafEnterpriseException si erreur 
     */
    public static void updateLastModifParams(Long pApplicationId, String pLastUser, Date pLastUpdate) throws JrafEnterpriseException {
        // Initialisation du DAO
        ApplicationDAOImpl appliDAO = ApplicationDAOImpl.getInstance();
        // La session
        ISession session = null;
        try {
            session = PERSISTENTPROVIDER.getSession();
            // On rcuprer l'application
            ApplicationBO appliBO = (ApplicationBO)appliDAO.get(session, pApplicationId);
            // On modifie les paramtres concernant les dernires modifications
            appliBO.setLastUpdate(pLastUpdate);
            appliBO.setLastUser(pLastUser);
            // On sauvegarde en base
            appliDAO.save(session, appliBO);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ApplicationFacade.class.getName() + ".updateLastModifParams");
        } finally {
            FacadeHelper.closeSession(session, ApplicationFacade.class.getName() + ".updateLastModifParams");
        }

    }
}
