//Source file: D:\\CC_VIEWS\\SQUALE_V0_0_ACT\\SQUALE\\SRC\\squaleCommon\\src\\com\\airfrance\\squalecommon\\enterpriselayer\\facade\\UserFacade.java

package com.airfrance.squalecommon.enterpriselayer.facade.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.provider.persistence.hibernate.facade.FacadeHelper;
import com.airfrance.jraf.spi.enterpriselayer.IFacade;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.component.ApplicationDAOImpl;
import com.airfrance.squalecommon.daolayer.profile.ProfileDAOImpl;
import com.airfrance.squalecommon.daolayer.profile.UserDAOImpl;
import com.airfrance.squalecommon.datatransfertobject.component.ProfileDTO;
import com.airfrance.squalecommon.datatransfertobject.component.UserDTO;
import com.airfrance.squalecommon.datatransfertobject.transform.component.ComponentTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.component.ProfileTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.component.UserTransform;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ApplicationBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.profile.ProfileBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.profile.UserBO;

/**
 */
public class UserFacade implements IFacade {

    /**
     * Persistence provider
     */
    private static final IPersistenceProvider PERSISTENTPROVIDER = PersistenceHelper.getPersistenceProvider();

    /**
     * Constructeur vide
     * @roseuid 42CBFFB20158
     */
    private UserFacade() {
    }

    /**
     * retourne une collection de application pour un utilisateur donn
     * @param pUser utilisateur donn
     * @return Collection de applicationDTO (lastAudit ne sera pas instanci)
     * @throws JrafEnterpriseException exception JRAF
     * @deprecated l'utilisateur logu recoit l'objet UserDTO qui contient la liste
     * de application.
     * @roseuid 42CBFFB2014C
     */
    public static Collection getApplicationList(UserDTO pUser) throws JrafEnterpriseException {

        // Initialisation du retour et des BO
        Collection applicationDTOs = null;
        UserBO userBO = new UserBO();
        Set keyCollection = null;

        ISession session = null;

        try {

            session = PERSISTENTPROVIDER.getSession();

            UserDAOImpl userDAO = UserDAOImpl.getInstance();

            userBO = (UserBO) userDAO.loadWithMatricule(session, pUser.getMatricule());

            // Recuperation des cles qui sont des applicationBO
            keyCollection = (userBO.getRights()).keySet();

            //ajout au retour des BO transformes en DTO
            Iterator iterator = keyCollection.iterator();
            while (iterator.hasNext()) {

                applicationDTOs.add(ComponentTransform.bo2Dto((ApplicationBO) iterator.next()));

            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".getApplicationList");
        } finally {
            FacadeHelper.closeSession(session, UserFacade.class.getName() + ".getApplicationList");
        }

        return applicationDTOs;
    }

    /**
     * Retourne le UserDTO en renseignant l'identifiant
     * @param pUser utilisateur donn
     * @param pAdmin indique si le privilge admin est plac
     * @return UserDTO avec l'identifiant 
     * @throws JrafEnterpriseException exception JRAF
     */
    public static UserDTO getUser(UserDTO pUser, Boolean pAdmin) throws JrafEnterpriseException {

        // Initialisation 
        UserBO userBO = null; // retour de la DAO
        UserDTO userDTO = null; // retour de la facade

        ISession session = null;

        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();

            UserDAOImpl userDAO = UserDAOImpl.getInstance();

            // traitement different si on initialise l'identifiant dans le UserDTO :
            // si l'id est connu, l'utilisateur est dj authentifi, donc pas besoin de vrifier
            // son mot de passe
            if (pUser.getID() < 0) {
                // rcupration du UserBO  partir du matricule
                userBO = (UserBO) userDAO.loadWithMatricule(session, pUser.getMatricule());
            } else {
                // rcupration du UserBO  partir de l'id
                Long userID = new Long(pUser.getID());
                userBO = (UserBO) userDAO.get(session, userID);
            }

            // Traitement du cas du user guest : on affecte un profil en fonction
            // du privilge administrateur
            if (userBO == null) {
                userBO = createDefaultUser(session, pUser.getMatricule(), pAdmin);
            }

            // transformation du UserBO en UserDTO
            userDTO = UserTransform.bo2Dto(userBO);
            // ajout des droits de l'utilisateurs sur les diffrentes applications
            setRights(session, userDTO, userBO, pAdmin);

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".getApplicationList");
        } finally {
            FacadeHelper.closeSession(session, UserFacade.class.getName() + ".getApplicationList");
        }

        return userDTO;
    }

    
    /**
     * Return the complete userDTO corresponding with the identifier and the password 
     * @param pUser The userDTO with the identifier and the password inside
     * @return UserDTO found 
     * @throws JrafEnterpriseException exception JRAF
     */
    public static UserDTO getUserByMatriculeAndPassword(UserDTO pUser) throws JrafEnterpriseException {
    
    
    
    UserBO userBO = null; 
    UserDTO userDTO = null; 

    ISession session = null;

    try {
        
        session = PERSISTENTPROVIDER.getSession();

        UserDAOImpl userDAO = UserDAOImpl.getInstance();

        
        userBO = (UserBO) userDAO.loadWithMatriculeAndPassword(session, pUser.getMatricule(), pUser.getPassword());
        
       
        userDTO = UserTransform.bo2Dto(userBO);

    } catch (JrafDaoException e) {
        FacadeHelper.convertException(e, UserFacade.class.getName() );
    } finally {
        FacadeHelper.closeSession(session, UserFacade.class.getName() );
    }

    return userDTO;
}
    
    /**
     * Permet d'ajouter des droits au UserDTO
     * Dans le cas d'un utilisateur avec un privilge admin,
     * tous les applications valids sont ajouts  l'utilisateur
     * @param pSession session hibernate
     * @param pUserDTO UserDTO de destination
     * @param pUserBO UserBO contenant les droits
     * @param pAdmin indique si l'utilisateur a les droits admin
     * @return UserDTO renseign entierement
     * @throws JrafEnterpriseException exception Jraf
     */
    private static UserDTO setRights(ISession pSession, UserDTO pUserDTO, UserBO pUserBO, Boolean pAdmin) throws JrafEnterpriseException {

        // Initialisation
        UserDTO userDTO = pUserDTO; // retour de la methode
        Map rights = new HashMap(); // map des droits sur un utilisateur
        List publicApplicationsBO = new ArrayList();
        List publicApplicationsDTO = new ArrayList();
        List applicationsInCreation = new ArrayList();
        ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance(); // initialisation de Dao

        try {
            // Un administrateur a accs  toutes les applications confirmes
            if (pAdmin.booleanValue()) {
                ApplicationBO rightKey = null;
                Long applicationId = null;
                Iterator applicationIterator = null;
                // on ne veut pas les applications  supprimer
                // Rcupration de toutes les applications valides
                applicationIterator = applicationDAO.findWhereStatus(pSession, ApplicationBO.VALIDATED).iterator();
                // Affectation des droits admin sur les applications valides
                fillRights(pSession, applicationIterator, ProfileBO.ADMIN_PROFILE_NAME, rights, null);
                // Rcupration de toutes les applications a valider
                applicationIterator = applicationDAO.findWhereStatus(pSession, ApplicationBO.IN_CREATION).iterator();
                // Affectation des droits pour l'utilisateur courant admin sur les applications  valider
                fillRights(pSession, applicationIterator, ProfileBO.ADMIN_PROFILE_NAME, rights, null);
            } else {
                // Initialisation de la map  partir des droits utilisateur
                // L'utilisation d'une map permet de surcharger si besoin les droits par dfaut accords
                // sur l'application publique
                Set rightBOs = pUserBO.getRights().keySet();
                Iterator applicationIterator = rightBOs.iterator();
                ApplicationBO rightKey = null;
                Long applicationId = null;
                // La liste des ids des applications appartenant  l'utilisateur
                String userAppli = "";
                while (applicationIterator.hasNext()) {
                    // Chargement de l'application
                    applicationId = new Long(((ApplicationBO) applicationIterator.next()).getId());
                    rightKey = (ApplicationBO) applicationDAO.get(pSession, applicationId);
                    userAppli += ", " + applicationId;
                    // visibilit uniquement sur les applications valides ou en cours de cration
                    if (rightKey.getStatus() == ApplicationBO.VALIDATED) {
                        rights.put(ComponentTransform.bo2Dto(rightKey), ProfileTransform.bo2Dto((ProfileBO) pUserBO.getRights().get(rightKey)));
                     } else if (rightKey.getStatus() == ApplicationBO.IN_CREATION) {
                        rights.put(ComponentTransform.bo2Dto(rightKey), ProfileTransform.bo2Dto((ProfileBO) pUserBO.getRights().get(rightKey)));
                        applicationsInCreation.add(ComponentTransform.bo2Dto(rightKey));
                    }
                }
                // Recupration des applications publiques qui n'appartiennent pas  l'utilisateur
                if(userAppli.length() > 0) {
                    // On supprime le premire virgule inutile
                    userAppli = userAppli.substring(1);
                }
                publicApplicationsBO = applicationDAO.findPublic(pSession, userAppli);
                applicationIterator = publicApplicationsBO.iterator();
                // Affectation des droits reader
                fillRights(pSession, applicationIterator, ProfileBO.READER_PROFILE_NAME, rights, publicApplicationsDTO);
                
            }
            userDTO.setProfiles(rights);
            userDTO.setApplicationsInCreation(applicationsInCreation);
            userDTO.setPublicApplications(publicApplicationsDTO);

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".setRights");
        }

        return userDTO;
    }

    /**
     * Affectation des droits
     * Le paramtre pRights est mis  jour pour contenir l'ensemble des applications avec leur droit
     * associ
     * @param pSession session
     * @param pApplicationIterator applications concernes
     * @param pProfile profil  utiliser
     * @param pRights droits  renseigner
     * @param pPublicAppli la liste des applications publiques ayant des rsultats
     * @throws JrafDaoException si erreur
     */
    static private void fillRights(ISession pSession, Iterator pApplicationIterator, String pProfile, Map pRights, List pPublicAppli) throws JrafDaoException {
        // Affectation dans la Map pour chaque application du profil administrateur
        ProfileDAOImpl profileDAO = ProfileDAOImpl.getInstance();
        ProfileDTO adminProfile = ProfileTransform.bo2Dto((ProfileBO) profileDAO.loadByKey(pSession, pProfile));
        ApplicationDAOImpl applicationDAO = ApplicationDAOImpl.getInstance();
        while (pApplicationIterator.hasNext()) {
            // Chargement de l'application
            Long applicationId = new Long(((ApplicationBO) pApplicationIterator.next()).getId());
            ApplicationBO rightKey = (ApplicationBO) applicationDAO.get(pSession, applicationId);
            if (rightKey.getStatus() != ApplicationBO.DELETED) {
                pRights.put(ComponentTransform.bo2Dto(rightKey), adminProfile);
                // La liste des applications publiques peut tre nulle dans le cas
                // o l'utilisateur est administrateur car dans ce cas on ne les diffrencie pas
                if (pPublicAppli != null && rightKey.getPublic()) {
                    pPublicAppli.add(ComponentTransform.bo2Dto(rightKey));
                }
            }
        }
    }

    /**
     * Cration ou mise  jour de l'utilisateur
     * La cration se fait avec un profil minimal, la mise  jour ne concerne que l'adresse email ou le nom
     * @param pUser utilisateur donn
     * @param pAdmin indique si le privilge admin est plac
     * @return UserDTO avec l'identifiant 
     * @throws JrafEnterpriseException exception JRAF
     */
    public static UserDTO createOrUpdateUser(UserDTO pUser, Boolean pAdmin) throws JrafEnterpriseException {
        // Initialisation 
        UserBO userBO = null; // retour de la DAO
        UserDTO userDTO = null; // retour de la facade
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            UserDAOImpl userDAO = UserDAOImpl.getInstance();
            // traitement different si on initialise l'identifiant dans le UserDTO :
            // si l'id est connu, l'utilisateur est dj authentifi, donc pas besoin de vrifier
            // son mot de passe
            if (pUser.getID() < 0) {
                // rcupration du UserBO  partir du matricule
                userBO = (UserBO) userDAO.loadWithMatricule(session, pUser.getMatricule());
            } else {
                // rcupration du UserBO  partir de l'id
                Long userID = new Long(pUser.getID());
                userBO = (UserBO) userDAO.get(session, userID);
            }
            // Traitement du cas du user guest : on affecte un profil en fonction
            // du privilge administrateur
            if (userBO == null) {
                userBO = createDefaultUser(session, pUser.getMatricule(), pAdmin);
            }
            // Mise  jour des infomations du BO en fonction du DTO
            userBO.setEmail(pUser.getEmail());
            userBO.setFullName(pUser.getFullName());
            userBO.setUnsubscribed(pUser.isUnsubscribed());
            // Sauvegarde dans la base
            userDAO.save(session, userBO);
            // transformation du UserBO en UserDTO
            userDTO = UserTransform.bo2Dto(userBO);
            // ajout des droits de l'utilisateurs sur les diffrentes applications
            setRights(session, userDTO, userBO, pAdmin);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".getApplicationList");
        } finally {
            FacadeHelper.closeSession(session, UserFacade.class.getName() + ".getApplicationList");
        }
        return userDTO;
    }

    /**
     * Cration d'un utilisateur par dfaut
     * L'utilisateur est cre avec le matricule pass en paramtre et le profil par dfaut
     * en fonction du droit admin (soit admin soit default)
     * @param pSession session
     * @param pMatricule matricule
     * @param pAdmin indique si le profil admin est requis
     * @return user cr
     * @throws JrafDaoException si erreur
     */
    static private UserBO createDefaultUser(ISession pSession, String pMatricule, Boolean pAdmin) throws JrafDaoException {
        UserBO userBO = new UserBO();
        userBO.setMatricule(pMatricule);
        ProfileDAOImpl profileDAO = ProfileDAOImpl.getInstance();
        UserDAOImpl userDAO = UserDAOImpl.getInstance();
        ProfileBO defaultProfile;
        if (pAdmin.booleanValue()) {
            defaultProfile = (ProfileBO) profileDAO.loadByKey(pSession, ProfileBO.ADMIN_PROFILE_NAME);
        } else {
            defaultProfile = (ProfileBO) profileDAO.loadByKey(pSession, ProfileBO.DEFAULT_PROFILE_NAME);
        }
        userBO.setDefaultProfile(defaultProfile);
        // Sauvegarde de l'utilisateur 
        userDAO.save(pSession, userBO);
        return userBO;
    }

    /**
     * @param pId l'id de l'application 
     * @param pProfile le profil utilisateur recherch pour cette application
     * @param pUnsubscribed true si on veut rcuprer aussi les utilisateur qui se sont dsabonns
     * de l'envoi automatique d'email.
     * @return l'ensemble des utilisateurs ayant <code>pProfile</code> comme droit sur l'application
     * @throws JrafEnterpriseException en cas d'checs
     */
    public static Collection getUsersWithEmails(Long pId, String pProfile, boolean pUnsubscribed) throws JrafEnterpriseException {
        Collection result = new ArrayList(); // retour de la facade
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            UserDAOImpl userDAO = UserDAOImpl.getInstance();
            // Obtention des users avec le profil pProfile sur l'application
            ProfileBO profileBO = ProfileDAOImpl.getInstance().loadByKey(session, pProfile);
            Collection applicationUsers = userDAO.findWhereApplicationAndProfileAndHaveEmails(session, pId, profileBO, pUnsubscribed);
            Iterator it = applicationUsers.iterator();
            // Transformation des userBO en DTO
            while (it.hasNext()) {
                UserBO userBO = (UserBO) it.next();
                UserDTO userDTO = UserTransform.bo2Dto(userBO);
                result.add(userDTO);
            }
        } catch (JrafDaoException e) {
            // Renvoi d'une exception
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".getManagers");
        } finally {
            // Fermeture de la session
            FacadeHelper.closeSession(session, UserFacade.class.getName() + ".getManagers");
        }

        return result;
    }

    /**
     * Obtention des utilisateurs avec le profil administrateur
     * @param pUnsubscribed true si on veut rcuprer aussi les utilisateur qui se sont dsabonns
     * de l'envoi automatique d'email.
     * @return utilisateurs avec le profil administrateur ou ayant le profil manager
     * sur l'application concerne
     * @throws JrafEnterpriseException si erreur
     */
    static public Collection getAdminsWithEmails(boolean pUnsubscribed) throws JrafEnterpriseException {
        Collection result = new ArrayList(); // retour de la facade
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            UserDAOImpl userDAO = UserDAOImpl.getInstance();
            // Obtention des utilisateurs avec le profil admin
            Collection adminUsers = userDAO.findWhereAdminAndHaveEmails(session, pUnsubscribed);
            Iterator it = adminUsers.iterator();
            // transformation du UserBO en UserDTO
            while (it.hasNext()) {
                UserDTO userDTO = UserTransform.bo2Dto((UserBO) it.next());
                result.add(userDTO);
            }
        } catch (JrafDaoException e) {
            // Renvoi d'une exception
            FacadeHelper.convertException(e, UserFacade.class.getName() + ".getAdminUsers");
        } finally {
            // Fermeture de la session
            FacadeHelper.closeSession(session, UserFacade.class.getName() + ".getAdminUsers");
        }
        return result;
    }
}
