package com.airfrance.squaleweb.applicationlayer.action.results.application;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.helper.AccessDelegateHelper;
import com.airfrance.jraf.spi.accessdelegate.IApplicationComponent;
import com.airfrance.squaleweb.applicationlayer.action.accessRights.DefaultAction;
import com.airfrance.squaleweb.applicationlayer.formbean.component.ApplicationListForm;
import com.airfrance.squaleweb.transformer.ApplicationListTransformer;
import com.airfrance.squaleweb.transformer.FactorsResultListTransformer;
import com.airfrance.welcom.struts.bean.WActionForm;
import com.airfrance.welcom.struts.transformer.WTransformerException;
import com.airfrance.welcom.struts.transformer.WTransformerFactory;

/**
 * Affichage de la liste des applications
 * Cette action permet d'extraire la liste des applications disponibles pour l'utilisateur
 * courant et de les restituer sous une forme synthtique
 */
public class AllApplicationsAction extends DefaultAction {

    /**
     * Affichage de la liste des applications non publiques
     * @param pMapping le mapping.
     * @param pForm le formulaire  lire.
     * @param pRequest la requte HTTP.
     * @param pResponse la rponse de la servlet.
     * @return l'action  raliser.
     */
    public ActionForward listNotPublic(ActionMapping pMapping, ActionForm pForm, HttpServletRequest pRequest, HttpServletResponse pResponse) {

        ActionForward forward;
        try {
            // Rcupration de la liste des applications
            forward = execute(pMapping, pForm, getUserApplicationWithResultsList(pRequest));
        } catch (Exception e) {
            ActionErrors errors = new ActionErrors();
            // Traitement factoris des erreurs
            handleException(e, errors, pRequest);
            saveMessages(pRequest, errors);
            forward = pMapping.findForward("failure");
        }
        //On est pass par un menu donc on rinitialise le traceur
        resetTracker(pRequest);
        return (forward);
    }

    /**
     * Affichage de la liste des applications publiques
     * @param pMapping le mapping.
     * @param pForm le formulaire  lire.
     * @param pRequest la requte HTTP.
     * @param pResponse la rponse de la servlet.
     * @return l'action  raliser.
     */
    public ActionForward listPublic(ActionMapping pMapping, ActionForm pForm, HttpServletRequest pRequest, HttpServletResponse pResponse) {

        ActionForward forward;
        try {
            // Rcupration de la liste des applications
            forward = execute(pMapping, pForm, getUserPublicApplicationWithResultsList(pRequest));
            // On indique qu'il s'agit d'applications publiques
            pRequest.setAttribute("isPublic", "true");
        } catch (Exception e) {
            ActionErrors errors = new ActionErrors();
            // Traitement factoris des erreurs
            handleException(e, errors, pRequest);
            saveMessages(pRequest, errors);
            forward = pMapping.findForward("failure");
        }
        //On est pass par un menu donc on rinitialise le traceur
        resetTracker(pRequest);
        return (forward);
    }

    /**
     * @param pMapping le mapping
     * @param pForm le formulaire
     * @param pAppli les applications
     * @return le forward
     * @throws WTransformerException si erreur
     * @throws JrafEnterpriseException si erreur
     */
    private ActionForward execute(ActionMapping pMapping, ActionForm pForm, List pAppli) throws WTransformerException, JrafEnterpriseException {
        ActionForward forward;
        ApplicationListForm applicationListForm = new ApplicationListForm();
        ArrayList applicationsList = new ArrayList();
        applicationsList.addAll(pAppli);
        applicationListForm.setList(applicationsList);
        List applications = (List) (WTransformerFactory.formToObj(ApplicationListTransformer.class, applicationListForm)[0]);
        if (applications.isEmpty()) {
            // S'il n'y a pas d'applications, on redirige vers une page ddie
            forward = pMapping.findForward("empty");
        } else {
            List results = new LinkedList();
            // Rcupration des rsultats  partir de l'Application Component
            IApplicationComponent ac = AccessDelegateHelper.getInstance("Results");
            Object[] paramIn = { applications, null };
            // Rcupration des rsultats pour chacun des applications concernes
            List applicationResults = ((List) ac.execute("getApplicationResults", paramIn));
            // Transformation des rsultats avant leur affichage par la couche
            // welcom
            WTransformerFactory.objToForm(FactorsResultListTransformer.class, (WActionForm) pForm, new Object[] { applications, applicationResults });
            forward = pMapping.findForward("factors");
        }
        return forward;
    }

}
