package com.airfrance.squaleweb.applicationlayer.formbean.creation;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionMapping;

import com.airfrance.squalecommon.datatransfertobject.component.AuditDTO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squaleweb.applicationlayer.formbean.RootForm;
import com.airfrance.squaleweb.applicationlayer.formbean.access.AccessListForm;
import com.airfrance.squaleweb.applicationlayer.formbean.component.AuditForm;
import com.airfrance.squaleweb.applicationlayer.formbean.config.ServeurForm;
/**
 * Form bean for a Struts application.
 * 
 * @version 1.0
 * @author
 */
public class CreateApplicationForm extends RootForm {

    /** Les accs utilisateur */
    private AccessListForm mAccessListForm;
    
    /**
     * Prcise le site auquel appartient l'application
     */
    private ServeurForm mServeurForm = new ServeurForm();
    /**
     * Prcise si les audits sont raliss uniquement sur demande
     */
    private boolean mMilestone = false;
    /**
     * L'audit de jalon si l'application en a un programm
     */
    private AuditForm mMilestoneAudit = new AuditForm();

    /**
     * Dlai de purge par dfaut
     */
    public static final int DEFAULT_PURGE_DELAY = 180;
    /**
     * Dlai de purge par dfaut
     */
    public static final int DEFAULT_AUDIT_FREQUENCY = 15;
    /**
     * Dlai entre deux purges de l'application
     */
    private int mPurgeDelay = DEFAULT_PURGE_DELAY;
    /**
     * Dlai entre deux purges de l'application
     */
    private int mAuditFrequency = DEFAULT_AUDIT_FREQUENCY;
    /**
     * Status de l'application
     */
    private int mStatus;

    /**
     * Liste des projets de l'application
     */
    private List mProjects = new ArrayList();

    /**
     * Liste des droits sur l'application
     */
    private Map mRights = new HashMap();

    /**
     * Caractre publique 
     */
    private boolean mPublic;

    /** indique si l'application tait dja en production au moment de sa cration dans SQUALE */
    private boolean isInProduction = false;

    /** indique si l'application a t dvelopp en externe */
    private boolean externalDev = false;
    
    /** Date de la dernire modification */
    private Date mLastUpdate;
    
    /** L'utilisateur ayant fait la dernire modification */
    private String mLastUser;

    /**
     * @return le boolen indiquant si le dev a t fait en externe ou pas
     */
    public boolean getExternalDev() {
        return externalDev;
    }

    /**
     * @return le boolen indiquant si l'application tait dj en production au
     * moment de sa cration dans squale
     */
    public boolean getIsInProduction() {
        return isInProduction;
    }

    /**
     * @param pExternal le boolen indiquant si le dev a t fait en externe
     */
    public void setExternalDev(boolean pExternal) {
        externalDev = pExternal;
    }

    /**
     * @param pInProduction le boolen indiquant si l'application tait dj en production au
     * moment de sa cration dans squale
     */
    public void setIsInProduction(boolean pInProduction) {
        isInProduction = pInProduction;
    }

    /**
     * @return le dlai de purge
     */
    public int getPurgeDelay() {
        return mPurgeDelay;
    }

    /**
     * @param pPurgeDelay le dlai de purge (en jour)
     */
    public void setPurgeDelay(int pPurgeDelay) {
        mPurgeDelay = pPurgeDelay;
    }

    /**
     * @return la frquence d'audit en jours
     */
    public int getAuditFrequency() {
        return mAuditFrequency;
    }

    /**
     * @param pAuditFrequency la frquence d'audit en jours
     */
    public void setAuditFrequency(int pAuditFrequency) {
        mAuditFrequency = pAuditFrequency;
    }

    /**
     * @return la valeur de mMilestone.
     */
    public boolean isMilestone() {
        return mMilestone;
    }

    /**
     * @param pMilestone la nouvelle valeur de mMilestone.
     */
    public void setMilestone(boolean pMilestone) {
        mMilestone = pMilestone;
    }

    /**
     * @return la liste des projets de l'application
     */
    public List getProjects() {
        return mProjects;
    }

    /**
     * @param pProjects la liste des projets de l'application
     */
    public void setProjects(List pProjects) {
        mProjects = pProjects;
    }

    /**
     * @return la map des droits utilisateur / droits
     */
    public Map getRights() {
        return mRights;
    }

    /**
     * @param pMap la map des droits utilisateur / droits
     */
    public void setRights(Map pMap) {
        mRights = pMap;
    }

    /**
     * @param pStatus status
     */
    public void setStatus(int pStatus) {
        mStatus = pStatus;
    }

    /**
     * @return status
     */
    public int getStatus() {
        return mStatus;
    }

    /**
     * Initialisation des champs
     * Cette mthode permet de rinitialiser les champs autrement que
     * par la mthode reset classique car le formbean est utilis en
     * session et sous la forme d'un wizard
     */
    public void resetFields() {
        setApplicationId("-1");
        setApplicationName(null);
        mMilestone = false;
        mPurgeDelay = DEFAULT_PURGE_DELAY;
        mAuditFrequency = DEFAULT_AUDIT_FREQUENCY;
        mStatus = 0;
        mProjects = new ArrayList();
        mRights = new HashMap();
        mMilestoneAudit = new AuditForm();
        mMilestoneAudit.setType(AuditBO.MILESTONE);
        mServeurForm = null;
    }

    /**
     * @param pPublic caractre publique
     */
    public void setPublic(boolean pPublic) {
        mPublic = pPublic;
    }

    /**
     * 
     * @return caractre publique
     */
    public boolean isPublic() {
        return mPublic;
    }

    /** 
     * @see org.apache.struts.action.ActionForm#reset(org.apache.struts.action.ActionMapping, javax.servlet.http.HttpServletRequest)
     * {@inheritDoc}
     */
    public void reset(ActionMapping arg0, HttpServletRequest arg1) {
        super.reset(arg0, arg1);
        setMilestone(false);
        setPublic(false);
        setExternalDev(false);
        setIsInProduction(false);
    }

    /**
     * @return l'audit de jalon programm
     */
    public AuditForm getMilestoneAudit() {
        return mMilestoneAudit;
    }

    /**
     * @param pAuditForm l'audit de jalon programm
     */
    public void setMilestoneAudit(AuditForm pAuditForm) {
        mMilestoneAudit = pAuditForm;
    }

    /**
     * Remet  zro l'audit de jalon.
     */
    public void resetMilestoneAudit() {
        mMilestoneAudit = new AuditForm();

    }

    /**
     * @param pAuditDTO l'audit sous forme dto
     */
    public void setMilestoneAudit(AuditDTO pAuditDTO) {
        AuditForm audit = new AuditForm(pAuditDTO);
        setMilestoneAudit(audit);
    }

    /**
     * @return la date de la dernire modification
     */
    public Date getLastUpdate() {
        return mLastUpdate;
    }

    /**
     * @param pDate la date de la dernire modification
     */
    public void setLastUpdate(Date pDate) {
        mLastUpdate = pDate;
    }

    /**
     * @return l'utilisateur ayant fait la dernire modification
     */
    public String getLastUser() {
        return mLastUser;
    }

    /**
     * @param pMatricule l'utilisateur ayant fait la dernire modification
     */
    public void setLastUser(String pMatricule) {
        mLastUser = pMatricule;
    }
    
    /**
     * Mthode de validation pour vrifier que le nom mis  l'application ne contient pas de caractres spciaux.
     * La liste des caractres spciaux est dfini par le pattern situ dans la classe mre
     * @param pMapping le mapping hibernate
     * @param pRequest la requete
     */
    public void wValidate(ActionMapping pMapping,HttpServletRequest pRequest){
        if(getApplicationName().trim().length() == 0) {
            addError("applicationName", new ActionError("error.application.name.empty"));
        } else if(!isAValidName(getApplicationName())){
            addError("applicationName", new ActionError("error.name.containsInvalidCharacter"));
        }
    }
    /**
     * @return le formulaire du Serveur
     */
    public ServeurForm getServeurForm() {
        return mServeurForm;
    }

    /**
     * @param pServeurForm le formulaire du Serveur
     */
    public void setServeurForm(ServeurForm pServeurForm) {
        mServeurForm = pServeurForm;
    }

    /**
     * @param pForm les accs
     */
    public void setAccessListForm(AccessListForm pForm) {
       mAccessListForm = pForm;
    }

    /**
     * @return les accs utilisateur
     */
    public AccessListForm getAccessListForm() {
        return mAccessListForm;
    }

}
