package com.airfrance.squaleweb.tagslib;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.struts.util.ResponseUtils;

import com.airfrance.squaleweb.applicationlayer.tracker.TrackerStructure;
import com.airfrance.squaleweb.resources.WebMessages;
import com.airfrance.squaleweb.util.SqualeWebConstants;

/**
 */
public class TrackerTag extends TagSupport {

    /**
     * Constante indiquant le nombre d'lments du traceur qui sera affich
     */
    public final static int TRACKER_ELEMENTS_NUMBER = 5;

    /** pour indiquer si on vient directement d'une vue composant ou pas */
    private String directWay;

    /** la cl de la pratique */
    private String practiceKey;

    /** la cl du facteur */
    private String factorKey;

    /** le nom du projet */
    private String projectName;

    // un des 2 ids est obligatoire 

    /**
     * l'id du projet 
     */
    private String projectId;

    /**
     * l'id de l'application 
     */
    private String applicationId;

    /** le nom du composant */
    private String componentName;

    /** id de l'audit courant */
    private String currentAuditId;

    /** id de l'audit prcdent */
    private String previousAuditId;

    /**
     * @see javax.servlet.jsp.tagext.TagSupport#doEndTag()
     * {@inheritDoc}
     * Mthode de lancement du tag
     */
    public int doStartTag() throws JspException {
        // Publie
        ResponseUtils.write(pageContext, getHtmlCode());
        return SKIP_BODY;
    }

    /** sert pour mettre les ids concernant les audits */
    private String mAuditLink = "";

    /** sert pour mettre les ids concernant les projets et applis */
    private String mProjectOrApplicationIdLink = "";

    /**
     * @return le code html
     */
    private String getHtmlCode() {
        if (projectId == null && applicationId != null) {
            mProjectOrApplicationIdLink = "&applicationId=" + applicationId;
        } else {
            mProjectOrApplicationIdLink = "&projectId=" + projectId;
        }
        if (currentAuditId != null) {
            mAuditLink += "&currentAuditId=" + currentAuditId;
        }
        if (previousAuditId != null) {
            mAuditLink += "&previousAuditId=" + previousAuditId;
        }
        String result = " <div id=\"traceur\"> ";
        //la session ncessaire pour pouvoir les rcuprer
        HttpSession session = pageContext.getSession();
        // dans ces cas on ne se pose pas de questions 
        // on affiche directement le traceur historique
        if (directWay == null || directWay.equals("false")) {
            result += buildHtmlStringForHistTracker((List) session.getAttribute(SqualeWebConstants.TRACKER_HIST));
        } else {
            // on teste ce cas car l'affichage de toutes les pages
            // jsp est presque identique sauf dans ce cas particulier. 
            if (practiceKey != null) {
                String practiceName = WebMessages.getString((HttpServletRequest) pageContext.getRequest(), practiceKey);
                String link = "/squale/project.do?action=practice&which=" + practiceKey + mProjectOrApplicationIdLink + "&whichParent=" + factorKey + mAuditLink;
                String linkFactor = "/squale/project.do?action=factor&which=" + factorKey + mProjectOrApplicationIdLink + mAuditLink;
                String linkName = WebMessages.getString((HttpServletRequest) pageContext.getRequest(), factorKey);
                result += "<a href=\"/squale/project.do?action=summary\" > < " + projectName + " > </a> &gt";
                result += "<a href=\"" + linkFactor + "\"> < " + linkName + "> </a> &gt";
                result += "<a href=\"" + link + "\" > <" + practiceName + "> </a> ";
            } else { // sinon c'est forcment le traceur pour les composants
                result += buildHtmlStringForComponentTracker((List) session.getAttribute(SqualeWebConstants.TRACKER_COMPONENT));
            }
        }
        result += "</div> ";
        return result;
    }

    /**
     * @param list la liste des composants  tracer
     * @return le code html correspondant au traceur
     */
    private String buildHtmlStringForComponentTracker(List list) {
        String result = "";
        String name = "";
        int i;
        // on doit afficher au maximum TRACKER_ELEMENTS_NUMBER elements, et le dernier en non cliquable
        int startIndex = Math.max(0, list.size() - (1 + TRACKER_ELEMENTS_NUMBER));
        // (list.size -1) - startIndex < (TRACKER_ELEMENTS_NUMBER)
        for (i = startIndex; i < list.size() - 1; i++) {
            TrackerStructure structure = (TrackerStructure) list.get(i);
            name = structure.getDisplayName();
            result += structure.getLink() + mProjectOrApplicationIdLink + mAuditLink + "\" />" + name + "</a> &gt; ";
        }
        // le dernier en non cliquable
        if (i == list.size() - 1) {
            TrackerStructure structure = (TrackerStructure) list.get(i);
            // sans les "("
            StringTokenizer st = new StringTokenizer(structure.getDisplayName(), "(");
            if (st.hasMoreTokens()) {
                String last = st.nextToken();
                if (!last.equals(name)) {
                    result += last;
                }
            }
        }
        return result;
    }

    /**
     * Code factoris car la construction du code
     * @param list les lments du traceur
     * @return le code html
     */
    private String buildHtmlStringForHistTracker(List list) {
        String result = "";
        if (list == null) {
            list = new ArrayList();
        }
        TrackerStructure ts;
        // on limite l'affichage aux TRACKER_ELEMENTS_NUMBER 
        //premiers lments
        for (int i = Math.max(0, list.size() - TRACKER_ELEMENTS_NUMBER); i < list.size() - 1; i++) {
            ts = (TrackerStructure) list.get(i);
            if (!"".equals(ts.getDisplayName())) {
                result += "<a href=\"" + ts.getLink() + mAuditLink + "\" > " + ts.getDisplayName()  +" </a> &gt;";
            }
        }
        //gestion du dernier lment
        ts = (TrackerStructure) list.get(list.size() - 1);
        // dans ce cas il n'y a pas de nom de composant donc
        // le dernier lment sans ">"  la fin et non cliquable
        result += ts.getDisplayName();
        return result;
    }

    /**
     * @see javax.servlet.jsp.tagext.TagSupport#doEndTag()
     * {@inheritDoc}
     * Mthode de lancement du tag
     */
    public int doEndTag() throws JspException {
        return EVAL_PAGE;
    }

    /**
     * @return l'lment indiquant si on vient directement d'une vue composant ou pas 
     */
    public String getDirectWay() {
        return directWay;
    }

    /**
     * @return la cl du facteur
     */
    public String getFactorKey() {
        return factorKey;
    }

    /**
     * @return la cl de la pratique
     */
    public String getPracticeKey() {
        return practiceKey;
    }

    /**
     * @return le nom du projet
     */
    public String getProjectName() {
        return projectName;
    }

    /**
     * @param newWay la nouvelle valeur du chemin
     */
    public void setDirectWay(String newWay) {
        directWay = newWay;
    }

    /**
     * @param newFactorKey la nouvelle cl de facteur
     */
    public void setFactorKey(String newFactorKey) {
        factorKey = newFactorKey;
    }

    /**
     * @param newPracticeKey la nouvelle cl de facteur
     */
    public void setPracticeKey(String newPracticeKey) {
        practiceKey = newPracticeKey;
    }

    /**
     * @param newName le nouveau nom
     */
    public void setProjectName(String newName) {
        projectName = newName;
    }

    /**
     * @return le nom du composant
     */
    public String getComponentName() {
        return componentName;
    }

    /**
     * @param newName le nouveau nom du composant
     */
    public void setComponentName(String newName) {
        componentName = newName;
    }

    /**
     * @return l'id de l'application
     */
    public String getApplicationId() {
        return applicationId;
    }

    /**
     * @return l'id du projet
     */
    public String getProjectId() {
        return projectId;
    }

    /**
     * @param newApplicationId le nouvel id pour l'application
     */
    public void setApplicationId(String newApplicationId) {
        applicationId = newApplicationId;
    }

    /**
     * @param newProjectId le nouvel id du projet
     */
    public void setProjectId(String newProjectId) {
        projectId = newProjectId;
    }

    /**
     * @return l'id de l'audit courant
     */
    public String getCurrentAuditId() {
        return currentAuditId;
    }

    /**
     * @return l'id de l'audit prcdent
     */
    public String getPreviousAuditId() {
        return previousAuditId;
    }

    /**
     * @param pCurrentAuditId l'id de l'audit courant
     */
    public void setCurrentAuditId(String pCurrentAuditId) {
        currentAuditId = pCurrentAuditId;
    }

    /**
     * @param pPreviousAuditId l'id de l'audit prcdent
     */
    public void setPreviousAuditId(String pPreviousAuditId) {
        previousAuditId = pPreviousAuditId;
    }

}
