//Source file: D:\\cc_views\\squale_v0_0_act\\squale\\src\\squalix\\src\\com\\airfrance\\squalix\\tools\\compiling\\java\\compiler\\xml\\XMLCompiler.java

package com.airfrance.squalix.tools.compiling.java.compiler.xml;


import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;
import org.apache.tools.ant.types.Path;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.tools.compiling.CompilingMessages;
import com.airfrance.squalix.tools.compiling.java.JCompilingTask;
import com.airfrance.squalix.tools.compiling.java.beans.JXMLProject;
import com.airfrance.squalix.tools.compiling.java.configuration.JCompilingConfiguration;


/**
 * @author m400832 (by rose)
 * @version 1.0
 */
public class JXMLCompiler {
    
    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(
        JXMLCompiler.class);
    
    /**
     * Projet  compiler. 
     */
    private JXMLProject mProject = null;
    
        
    /**
     * Constructeur.
     * @param pProject projet  compiler.
     */
    public JXMLCompiler(JXMLProject pProject) {
        mProject = pProject;
    }
    
    /**
     * Cette mthode lance la procdure de compilation.
     * @throws Exception en cas d'erreur lors de l'excution du script ANT.
     * @see #doCompilation()
     */
    public void execute() throws Exception{
        LOGGER.trace(CompilingMessages.getString("logs.task.entering_method"));
        doCompilation();
    }
    
    /**
     * Cette mthode lance effectivement la compilation.
     * @throws Exception exception lors de la compilation par ANT.
     */
    private void doCompilation() throws Exception{
        LOGGER.trace(CompilingMessages.getString("logs.task.entering_method"));
        
        /*
         * Cration d'un descripteur de fichier avec un chemin UNIX. 
         */
        File buildFile = new File((mProject.getPath()).replaceAll(
            JCompilingConfiguration.WINDOWS_SEPARATOR + 
            JCompilingConfiguration.WINDOWS_SEPARATOR,
            JCompilingConfiguration.UNIX_SEPARATOR));
        
        /*
         * La procdure n'est lance que si le fichier XML est trouv. 
         */    
        if(buildFile.exists() && buildFile.isFile()){
            /*
             * Initialisation du projet ANT.
             */
            Project project = new Project();
            project.setCoreLoader(null);
            project.init();
            
            /*
             * Proprits du projet ANT.
             */
            project.setUserProperty(CompilingMessages.getString(
                "java.compiling.ant.key.ant.version"), CompilingMessages
                .getString("java.compiling.ant.value.ant.version"));
            project.setUserProperty(CompilingMessages.getString(
                "java.compiling.ant.key.ant.file"), 
                buildFile.getAbsolutePath());
            
            
            /* mode keep-going  false */    
            project.setKeepGoingMode(false);
                   
            
            // Ajout du listener pour les traces
            project.addBuildListener(mProject.getListener());
            /*
             * Parsing du fichier xml.
             */
            ProjectHelper helper = ProjectHelper.getProjectHelper();
            project.addReference(CompilingMessages.getString(
                "java.compiling.ant.key.ant.project_helper"), helper);
            helper.parse(project, buildFile);
            
            /*
             *Rcupration du classpath dans le fichier build.xml et du classes_dir.   
             */
            antClasspath(project);
 
            /*
             * Ajout de la tche  lancer.
             */
            Vector targets = new Vector();
            /* si une target a t dfinie dans le JXMLProject */
            if(null != mProject.getTarget() && !"".equals(mProject.getTarget())){
                targets.addElement((String)mProject.getTarget());
            
            /* sinon on ajoute la target par dfaut du projet ANT */
            } else {
                targets.addElement(project.getDefaultTarget());
            }
            
            /*
             * Excution de la tche ANT.
             */
            project.executeTargets(targets);
    
            /* mnage */
            project = null;
            targets = null;
            helper = null;
            
        } else {
            /* exception lance si le fichier xml n'est pas trouv */
            throw new FileNotFoundException(CompilingMessages.getString(
                "java.exception.compiling.ant.build_file_not_found"));
        }
        
    }
    
    /**
     * Getter.
     * @return un projet JXMLProject.
     */
    public JXMLProject getXMLProject(){
        return mProject;
    }
    
    
        
    
    /**
     * Mthode qui permet de recuperer le classpath et de redefinir prcisment l'emplacement
     * des fichiers compils. Pour que cette mthode marche, il faut que l'utilisateur ait 
     * dfini dans le fichier build.xml : Une proprit nomme SqualeClassesDir qui indique
     * l'emplacement des fichiers compils et une rfrence gnrant le classpath portant
     * l'identifiant SqualeClasspath 
     * @param antProject projet ant dans lequel on effectue la recherche
     */
    private void antClasspath (Project antProject){
        JCompilingTask compilingTask = (JCompilingTask) mProject.getListener();
        String classesDir = antProject.getProperty("SqualeClassesDir");
        File baseDir = antProject.getBaseDir(); 
        File test = new File(classesDir);
        if(!test.isAbsolute()){
        //if(!classesDir.startsWith("/")){
            classesDir = baseDir.getAbsolutePath()+"/"+classesDir;
        }
        List classesDirs = new ArrayList();
        classesDirs.add(classesDir);
        compilingTask.getData().putData(TaskData.CLASSES_DIRS,classesDirs);
        Path antClassPath = (Path)antProject.getReference("SqualeClasspath");
        String classpath= antClassPath.toString();
        String[] allpath = classpath.split(":");
        for (int i=0 ;  i < allpath.length; i++){
            int index = allpath[i].indexOf("${");
            if (index != -1){
                LOGGER.warn(CompilingMessages.getString("java.exception.compiling.ant.classpath_warning")+ allpath[i]);
            }
        }
        classpath = classpath.replace(':',';');
        classpath = classpath +';'+ classesDir; 
        compilingTask.getData().putData(TaskData.CLASSPATH,classpath);
    }
}