package com.airfrance.squalix.tools.cpptest;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.daolayer.rulechecking.CppTestRuleSetDAOImpl;
import com.airfrance.squalecommon.datatransfertobject.rulechecking.CppTestRuleSetDTO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.IntegerMetricBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.CppTestTransgressionBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.RuleCheckingTransgressionBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.RuleCheckingTransgressionItemBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.RuleBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.cpptest.CppTestRuleSetBO;

/**
 * Persistance des rsultats CppTest
 */
public class CppTestPersistor {

    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(CppTestPersistor.class);

    /**
     * Sauvegarde des rsultats
     * @param pSession session
     * @param pProjectBO projet
     * @param pAuditBO audit
     * @param pResults rsultats
     * @param pRuleSet ruleset
     * @throws JrafDaoException si erreur
     */
    public void storeResults(ISession pSession, ProjectBO pProjectBO, AuditBO pAuditBO, Map pResults, CppTestRuleSetDTO pRuleSet) throws JrafDaoException {
        // On rcupre le jeu de rgles associ au projet
        CppTestRuleSetBO ruleset = (CppTestRuleSetBO) CppTestRuleSetDAOImpl.getInstance().get(pSession, new Long(pRuleSet.getId()));
        // Cration de la transgression
        CppTestTransgressionBO transgression = new CppTestTransgressionBO();
        transgression.setAudit(pAuditBO);
        transgression.setComponent(pProjectBO);
        transgression.setRuleSet(ruleset);
        transgression.setTaskName("task.cpptest.name");
        // On parcourt les rgles connues dans le ruleset
        Iterator ruleCodes = ruleset.getRules().values().iterator();
        while (ruleCodes.hasNext()) {       
            RuleBO rule = (RuleBO) ruleCodes.next();
            Collection details = (Collection) pResults.get(rule.getCode());
            int nbOcc = 0;
            // Si le parsing n'a pas donn de rsultat, on place 0 comme
            // nombre de transgression
            if (details!=null) {
                nbOcc = details.size();
                // On parcourt le dtail des transgressions
                int cpt = RuleCheckingTransgressionBO.MAX_DETAILS;
                for (Iterator detailIt = details.iterator(); detailIt.hasNext() && cpt > 0; cpt--) {
                    RuleCheckingTransgressionItemBO item = (RuleCheckingTransgressionItemBO) detailIt.next();
                    item.setRule(rule);
                    transgression.getDetails().add(item);
                }
            }
            // On ajoute une mtrique de type Integer pour chaque rgle transgresse
            // avec 0 comme valeur par dfaut
            IntegerMetricBO metric = new IntegerMetricBO();
            metric.setName(rule.getCode());
            metric.setValue(nbOcc);
            metric.setMeasure(transgression);
            transgression.putMetric(metric);
        }
        // On parcourt les rgles non trouves dans le ruleset
        ruleCodes = pResults.keySet().iterator();
        while (ruleCodes.hasNext()) {
            String ruleCode = (String) ruleCodes.next();
            // Chaque rgle non trouve est signale comme
            // manquante
            if (false==ruleset.getRules().containsKey(ruleCode)) {
                LOGGER.warn(CppTestMessages.getString("rule.ignored", ruleCode));
            }
        }
        // Sauvegarde des donnes dans la base
        MeasureDAOImpl.getInstance().create(pSession, transgression);
    }

}
