/*
 * $Id: ClassMetricsContainer.java 8 2008-02-21 16:01:23Z bellingard $
 *
 * (C) Copyright 2005 Diomidis Spinellis
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

package gr.spinellis.ckjm;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Map;

/**
 * A container of class metrics mapping class names to their metrics.
 * This class contains the metrics for all class's during the filter's
 * operation.  Some metrics need to be updated as the program processes
 * other classes, so the class's metrics will be recovered from this
 * container to be updated.
 *
 * @version $Revision: 1.9 $
 * @author <a href="http://www.spinellis.gr">Diomidis Spinellis</a>
 */
public class ClassMetricsContainer {

    /** The map from class names to the corresponding metrics */
    private HashMap m = new HashMap();

    /**
     * Return a class's metrics
     * @param name le nom de la classe
     * @return la mtrique correspondante
     */
    public ClassMetrics getMetrics(String name) {
        ClassMetrics cm = (ClassMetrics) m.get(name);
        if (cm == null) {
            cm = new ClassMetrics();
            m.put(name, cm);
        }
        return cm;
    }

    /**
     * Print the metrics of all the visited classes.
     * @param handler outputHandler
     */
    public void printMetrics(CkjmOutputHandler handler) {
        Set entries = m.entrySet();
        Iterator i;

        for (i = entries.iterator(); i.hasNext();) {
            Map.Entry e = (Map.Entry) i.next();
            ClassMetrics cm = (ClassMetrics) e.getValue();
            if (cm.isVisited() && (MetricsFilter.includeAll() || cm.isPublic())) {
                handler.handleClass((String) e.getKey(), cm);
            }
        }
    }
    /**
     * @return classes name
     */
    public Set getEntries() {
        return m.entrySet();
    }

}
