package com.airfrance.squale.builder.eclipse32.actions;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IAccessRule;
import org.eclipse.jdt.core.IClasspathAttribute;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaModelMarker;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import com.airfrance.squale.builder.eclipse32.exception.BuilderException;
import com.airfrance.squale.builder.eclipse32.messages.BuilderMessages;

/**
 * Compilation d'un workspace
 */
public class BuildWorkspace implements IPlatformRunnable {
	/**
	 * Logger
	 */
	private static final Log LOGGER = LogFactory.getLog(BuildWorkspace.class);

	/** Le workspace */
	private IWorkspace wspace;

	/** La version de java utilis pour compiler le projet */
	private String dialect = JavaCore.VERSION_1_5;

	/** La chemin vers les libraries de l'API sun */
	private Set<String> sunLibs = new HashSet<String>();

	/** La concatnation des classpaths des projets */
	private StringBuffer classpath = new StringBuffer("");
	
	/** La concatnation des rpertoires de compilation des projets */
	private StringBuffer outputDirs = new StringBuffer("");


	/** La liste des projets cres */
	private List<IProject> projects = new ArrayList<IProject>();

	/** Les patterns d'exclusion pour les entres de type SOURCE */
	private IPath[] exclusionPatterns = new IPath[0];

	/** Map des librairies utilisateur */
	private Map<IPath, IClasspathEntry[]> userLibs = new HashMap<IPath, IClasspathEntry[]>(
			0);

	/**
	 * Cration du workspace et compilation
	 * 
	 * @throws Exception
	 *             si erreur
	 */
	public Object run(Object args) throws Exception {
		try {
			// On rcupre le workspace donn par -data
			wspace = ResourcesPlugin.getWorkspace();

			// Rcupration des arguments
			String[] arguments = (String[]) args;
			// On vrifie les arguments de la commande
			if (1 == arguments.length) {
				if (arguments[0].equals(BuildWorkspaceConstants.HELP)) {
					// On ne fait qu'afficher l'aide
					LOGGER.info(BuilderMessages.getString("info.help"));
				} else if (arguments[0].equals(BuildWorkspaceConstants.VERSION)) {
					// On ne fait qu'afficher la version
					LOGGER.info(BuilderMessages.getString("info.version"));
				}

			} else if (0 == arguments.length || !verifyMandatoryArgs(arguments)) {
				throw new BuilderException(BuilderMessages
						.getString("exception.builder.usage"));
			} else {
				// On rcupre les arguments de la commande
				setArgs(arguments);

				// On construit les projets de type Java
				for (int i = 0; i < projects.size(); i++) {
					if (null != projects.get(i).getNature(JavaCore.NATURE_ID)) {
						// Il s'agit bien d'un projet java
						addProject(projects.get(i));
					}
				}
				wspace.build(IncrementalProjectBuilder.FULL_BUILD, null);

				// On log les erreurs
				List<IMarker> compilationPb = new ArrayList<IMarker>(0);
				for (int i = 0; i < wspace.getRoot().getProjects().length; i++) {
					// IResource.DEPTH_ONE pour n'avoir que les markers
					// du projet.
					compilationPb.addAll(Arrays.asList(wspace.getRoot()
							.getProjects()[i].findMarkers(
							IJavaModelMarker.JAVA_MODEL_PROBLEM_MARKER, true,
							IResource.DEPTH_ONE)));
					compilationPb.addAll(Arrays.asList(wspace.getRoot()
							.getProjects()[i].findMarkers(
							IJavaModelMarker.BUILDPATH_PROBLEM_MARKER, true,
							IResource.DEPTH_ONE)));
					for (int err = 0; err < compilationPb.size(); err++) {
						logCompilationProblems(compilationPb.get(err));
					}
				}
				// On retourne le classpath
				LOGGER.info(BuildWorkspaceConstants.CLASSPATH_KEY
						+ classpath.toString());
				// On retourne la liste des rpertoires de compilation
				LOGGER.info(BuildWorkspaceConstants.OUTPUTS_KEY
						+ outputDirs.toString());
			}
		} catch (Exception e) {
			// On log l'erreur pour la rcuprer en dehors du fichier.log
			LOGGER.error(e);
		}
		return EXIT_OK;
	}

	/**
	 * Vrifie que tous les arguments obligatoires pour le bon droulement de la
	 * compilation sont prsents
	 * 
	 * @param arguments
	 *            les arguments  vrifier
	 * @return true si tous les arguments obligatoires sont prsents
	 */
	private boolean verifyMandatoryArgs(String[] arguments) {
		List<String> args = Arrays.asList(arguments);
		return args.containsAll(Arrays
				.asList(BuildWorkspaceConstants.MANDATORY_OPTIONS));
	}

	/**
	 * Gestion des logs d'erreur. Log en fonction de la svrit du marqueur
	 * 
	 * @param marker
	 *            le marker d'erreur de compilation
	 * @throws Exception
	 *             si erreur
	 */
	private void logCompilationProblems(IMarker marker) throws Exception {
		Integer priority = (Integer) marker.getAttribute(IMarker.SEVERITY);
		if (null == priority) {
			priority = IMarker.SEVERITY_INFO;
		}
		String message = iMarker2String(marker);
		switch (priority) {
		// niveau erreur
		case IMarker.SEVERITY_ERROR:
			LOGGER.error(message);
			break;
		// niveau warning
		case IMarker.SEVERITY_WARNING:
			LOGGER.warn(message);
			break;
		// niveau info
		case IMarker.SEVERITY_INFO:
			LOGGER.info(message);
			break;
		default:
			// info par dfaut
			LOGGER.info(message);
			break;
		}
	}

	/**
	 * toString() d'un marqueur
	 * 
	 * @param marker
	 *            le marqueur  formater
	 * @return le message correspondant au marqueur donn en paramtre sous la
	 *         forme chemin du fichier: message (line : 2)
	 * @throws Exception
	 *             si erreur
	 */
	private String iMarker2String(IMarker marker) throws Exception {
		StringBuffer result = new StringBuffer();
		String location = (marker.getResource().getFullPath()).toOSString();
		if (location.length() > 0) {
			result.append(location);
			result.append(": ");
		}
		result.append((String) marker.getAttribute(IMarker.MESSAGE));
		Integer line = (Integer) marker.getAttribute(IMarker.LINE_NUMBER);
		if (null != line && line.intValue() > 0) {
			result.append(" (line:  ");
			result.append(line);
			result.append(")");
		}
		return result.toString();
	}

	/**
	 * Rcupration des arguments et initialisation des attributs
	 * 
	 * @param args
	 *            les arguments
	 */
	private void setArgs(String[] args) throws BuilderException {
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals(BuildWorkspaceConstants.DIALECT)) {
				setDialect(args, ++i);
			} else if (args[i]
					.equals(BuildWorkspaceConstants.LIST_PROJECTS_PATH)) {
				setProjectsPath(args, ++i);
			} else if (args[i].equals(BuildWorkspaceConstants.SUN_LIBRAIRIES)) {
				sunLibs.addAll(getOptionPaths(getOptionValue(args, ++i,
						"exception.options.sunLibs")));
			} else if (args[i].equals(BuildWorkspaceConstants.VAR_LIBRAIRIES)) {
				createVariables((getOptionValue(args, ++i)));
			} else if (args[i].equals(BuildWorkspaceConstants.USER_LIBRAIRIES)) {
				setUserLibs(getOptionValue(args, ++i));
			} else if (args[i]
					.equals(BuildWorkspaceConstants.EXCLUSION_PATTERNS)) {
				setExcludedPatterns(getOptionValue(args, ++i));
			} else {
				throw new BuilderException(BuilderMessages.getString(
						"exception.option.unknown", new String[] { args[i] }));
			}
		}
	}

	private void setUserLibs(String optionValue) throws BuilderException {
		// Si l'option est renseigne
		if (optionValue.length() > 0) {
			// On spare les libraries
			String[] userLibs = optionValue
					.split(BuildWorkspaceConstants.OPTION_VAR_SEP_REGEXP);
			// Pour toutes les libraries, on rcupre les chemins associs
			for (int i = 0; i < userLibs.length; i++) {
				String[] splitLibs = userLibs[i]
						.split(BuildWorkspaceConstants.OPTION_VAR_LIST_SEP);
				if (splitLibs.length != 2) {
					// Erreur dans l'appel
					throw new BuilderException("exception.options.userLibs");
				}
				String userLibName = splitLibs[0].trim();
				IPath userLibPath = new Path(JavaCore.USER_LIBRARY_CONTAINER_ID
						+ "/" + userLibName);
				// On rcupre les chemins associs  la librairie
				// et on cre le tableau des librairies associes.
				String[] libsPaths = splitLibs[1]
						.split(BuildWorkspaceConstants.OPTION_LIST_SEP);
				final IClasspathEntry[] entries = new IClasspathEntry[libsPaths.length];
				for (int l = 0; l < libsPaths.length; l++) {
					entries[l] = JavaCore.newLibraryEntry(
							new Path(libsPaths[l]), null, null);
				}
				this.userLibs.put(userLibPath, entries);
			}
		}
	}

	/**
	 * Modifie la liste des patterns  exclure
	 * 
	 * @param optionValue
	 *            la liste des patterns spars par ";"
	 */
	private void setExcludedPatterns(String optionValue) {
		// On spare les patterns
		String[] patterns = optionValue
				.split(BuildWorkspaceConstants.OPTION_LIST_SEP);
		exclusionPatterns = new IPath[patterns.length];
		for (int i = 0; i < patterns.length; i++) {
			exclusionPatterns[i] = new Path(patterns[i]);
		}
	}

	/**
	 * Crer les variables donnes en paramtre
	 * 
	 * @param args
	 *            les variables donnes en paramtre
	 * @throws BuilderException
	 *             si erreur dans l'appel de la commande
	 */
	private void createVariables(String args) throws BuilderException {
		// Si l'option est renseigne
		if (args.length() > 0) {
			// On spare les variables
			String[] vars = args
					.split(BuildWorkspaceConstants.OPTION_VAR_SEP_REGEXP);
			// Pour toutes les variables, on rcupre le chemin associ
			for (int i = 0; i < vars.length; i++) {
				String[] splitVar = vars[i]
						.split(BuildWorkspaceConstants.OPTION_VAR_LIST_SEP);
				if (splitVar.length != 2) {
					// Erreur dans l'appel
					throw new BuilderException(BuilderMessages.getString("exception.options.vars.libs"));
				}
				try {
					// On rcupre le chemin associ  la variable
					IPath libPath = new Path(splitVar[1]);
					File libFile = libPath.toFile();
					if (!libFile.exists()) {
						// Warning et on ne cre pas la variable
						LOGGER.warn(BuilderMessages.getString(
								"exception.var.not_correct", new String[] {
										splitVar[0].trim(),
										libFile.getAbsolutePath() }));
					} else {
						// On cre la variable
						JavaCore.setClasspathVariable(splitVar[0].trim(),
								libPath, null);
					}
				} catch (JavaModelException e) {
					throw new BuilderException(BuilderMessages.getString(
							"exception.variable.creation", splitVar[0].trim()));
				}
			}
		}
	}

	/**
	 * Cre les librairies utilisateur
	 * 
	 * @param optionValue
	 *            la valeur de l'option -userLibs
	 * @throws BuilderException
	 *             si erreur
	 */
	private void createUserLibs(IJavaProject javaProject)
			throws BuilderException {
		for (Iterator<IPath> it = userLibs.keySet().iterator(); it.hasNext();) {
			// On cre les container
			final IPath currentLib = it.next();
			try {
				JavaCore.setClasspathContainer(currentLib,
						new IJavaProject[] { javaProject },
						new IClasspathContainer[] { new IClasspathContainer() {
							public IClasspathEntry[] getClasspathEntries() {
								return userLibs.get(currentLib);
							}

							public String getDescription() {
								return "User Library "
										+ currentLib.toOSString();
							}

							public int getKind() {
								return IClasspathContainer.K_SYSTEM;
							}

							public IPath getPath() {
								return currentLib;
							}
						} }, null);
			} catch (JavaModelException e) {
				throw new BuilderException(BuilderMessages.getString(
						"exception.lib.creation", currentLib));
			}
		}
	}

	/**
	 * Retourne la valeur lie  une option dans l'appel de la commande (ex
	 * -sunLibs)
	 * 
	 * @param args
	 *            les arguments
	 * @param id
	 *            l'index de l'option  retourner
	 * @param malFormedException
	 *            le message d'erreur  renvoyer si l'option est mal form
	 * @return la valeur de l'option
	 * @throws BuilderException
	 *             si erreur dans l'appel
	 */
	private String getOptionValue(String[] args, int id,
			String malFormedException) throws BuilderException {
		if (id >= args.length) {
			throw new BuilderException(malFormedException);
		}
		return args[id];
	}

	/**
	 * Retourne la valeur lie  une option dans l'appel de la commande (ex
	 * -sunLibs)
	 * 
	 * @param args
	 *            les arguments
	 * @param id
	 *            l'index de l'option  retourner
	 * @return la valeur de l'option ou vide si l'option est sans valeur
	 */
	private String getOptionValue(String[] args, int id) {
		String result = "";
		if (id < args.length) {
			result = args[id];
		}
		return result;
	}

	/**
	 * Met en place et vrifie les chemins vers les projets  ajouter au
	 * workspace.
	 * 
	 * @param args
	 *            les arguments de la commande
	 * @param id
	 *            l'index de recherche
	 * @throws BuilderException
	 *             si erreur dans la construction de la commande
	 */
	private void setProjectsPath(String[] args, int id) throws BuilderException {
		if (id >= args.length) {
			throw new BuilderException("exception.options.projects");
		}
		String[] paths = args[id]
				.split(BuildWorkspaceConstants.OPTION_LIST_SEP);
		File currentPath;
		for (int i = 0; i < paths.length; i++) {
			currentPath = new File(paths[i].trim());
			if (!currentPath.exists()) {
				// Le projet indiqu n'existe pas
				String errorMsg = BuilderMessages.getString(
						"exception.project.not_found",
						new String[] { paths[i] });
				throw new BuilderException(errorMsg);
			}
			// On cre le projet
			IProject project = createProject(currentPath);
			// On l'ajoute  la liste des projets
			projects.add(project);
		}
	}

	/**
	 * Cre un projet au sens Eclipse
	 * 
	 * @param currentPath
	 *            le chemin du projet
	 * @return le projet eclipse
	 * @throws BuilderException
	 *             si erreur dans la cration
	 */
	private IProject createProject(File currentPath) throws BuilderException {
		IProject project = null;
		File projectDesc = new File(currentPath,
				IProjectDescription.DESCRIPTION_FILE_NAME);
		try {
			IProjectDescription projectDescription = wspace
					.loadProjectDescription(new FileInputStream(projectDesc));

			// On cre le projet dans le workspace grce au .project
			project = wspace.getRoot().getProject(projectDescription.getName());

			// On indique le chemin vers le projet  importer
			projectDescription.setLocation(new Path(currentPath
					.getAbsolutePath()));

			// On le crer !! Attention le projet ne doit pas exister dans le
			// workspace
			project.create(projectDescription, null);

			// Le projet doit tre ouvert pour la compilation
			if (project.exists() && !project.isOpen()) {
				project.open(null);
			}
		} catch (Exception ce) {
			ce.printStackTrace();
			// Le .project n'est pas conforme ou n'existe pas
			String errorMsg = BuilderMessages.getString(
					"exception.incorrect.project", new String[] { currentPath
							.getAbsolutePath() });
			throw new BuilderException(errorMsg, ce);
		}

		return project;
	}

	/**
	 * Ajoute le projet de type Java au workspace
	 * 
	 * @param project
	 *            le projet
	 * @throws BuilderException
	 *             si erreur durant l'ajout
	 */
	private void addProject(IProject project) throws BuilderException {
		// Cration du projet java  partir du IProject
		IJavaProject javaProject = JavaCore.create(project);
		if (null != javaProject) {
			try {
				// On change les options de compilation selon la version donne
				javaProject.setOption(JavaCore.COMPILER_COMPLIANCE, dialect);
				javaProject.setOption(
						JavaCore.COMPILER_CODEGEN_TARGET_PLATFORM, dialect);
				javaProject.setOption(JavaCore.COMPILER_SOURCE, dialect);
				// Ainsi que les paramtres pour la gestion du retour des
				// erreurs
				// On ne garde que les problme de compilation classique
				setJavaProjectOptions(javaProject);

				// On concatne le classpath du projet
				// On rcupre toutes les libraries  ajouter

				// On ajoute le rpertoire de compilation au classpath de retour
				File bin = new File(
						project.getLocation().toOSString(),
						javaProject
								.getOutputLocation()
								.toOSString()
								.replaceFirst("[/\\\\]" + project.getName(), ""));
				if (!bin.exists()) {
					bin.mkdirs();
				}
				addToClasspath(bin.getAbsolutePath());
				
				// On ajoute le rpertoire de compilation au classpath de retour
				// et  la liste  des rpertoires de compilation  renvoyer
				outputDirs.append(convertToUnixPath(bin.getAbsolutePath()));
				outputDirs.append(";");

				// On cre les libraries utilisateur pour ce projet
				createUserLibs(javaProject);

				// On parcours le classpath pour crer le classpath qui va
				// tre
				// retourner sous forme de log
				addToClasspath(javaProject.getRawClasspath(), javaProject);
				// On ajoute Les librairies de l'API sun (rt.jar, j2ee.jar,...)
				createAndAddLibsToClasspath(javaProject, sunLibs, false);

			} catch (Exception e) {
				throw new BuilderException(e);
			}
		}
	}

	/**
	 * Modifie les options par dfaut d'un projet Java afin d'ignorer certains
	 * warning
	 * 
	 * @param javaProject
	 *            le projet java
	 */
	private void setJavaProjectOptions(IJavaProject javaProject) {
		// Problme de javadoc
		javaProject.setOption(JavaCore.COMPILER_DOC_COMMENT_SUPPORT,
				JavaCore.DISABLED);
		// Annotation utilise comme super interface
		javaProject.setOption(JavaCore.COMPILER_PB_ANNOTATION_SUPER_INTERFACE,
				JavaCore.IGNORE);
		// Concatnation entre une String et un tableau de char
		javaProject.setOption(
				JavaCore.COMPILER_PB_CHAR_ARRAY_IN_STRING_CONCATENATION,
				JavaCore.IGNORE);
		// Type gnrique utilis dclar comme final
		javaProject.setOption(JavaCore.COMPILER_PB_FINAL_PARAMETER_BOUND,
				JavaCore.IGNORE);
		// Bloc "Finally" non complt
		javaProject.setOption(
				JavaCore.COMPILER_PB_FINALLY_BLOCK_NOT_COMPLETING,
				JavaCore.IGNORE);
		// Unreachable code
		javaProject.setOption(JavaCore.COMPILER_PB_HIDDEN_CATCH_BLOCK,
				JavaCore.IGNORE);
		// Id pour les classe srialise
		javaProject.setOption(JavaCore.COMPILER_PB_MISSING_SERIAL_VERSION,
				JavaCore.IGNORE);
		// Variable non initialise
		javaProject.setOption(JavaCore.COMPILER_PB_NO_EFFECT_ASSIGNMENT,
				JavaCore.IGNORE);
		// Import inutilis
		javaProject.setOption(JavaCore.COMPILER_PB_UNUSED_IMPORT,
				JavaCore.IGNORE);
		// variable non utilise
		javaProject.setOption(JavaCore.COMPILER_PB_UNUSED_LOCAL,
				JavaCore.IGNORE);
		// paramtre non utilis
		javaProject.setOption(JavaCore.COMPILER_PB_UNUSED_PARAMETER,
				JavaCore.IGNORE);
		// mthode ou champs priv non utilis
		javaProject.setOption(JavaCore.COMPILER_PB_UNUSED_PRIVATE_MEMBER,
				JavaCore.IGNORE);
		// Invocation non typ d'un paramtre typ (foo(X<String>) avec (X))
		javaProject.setOption(JavaCore.COMPILER_PB_UNCHECKED_TYPE_OPERATION,
				JavaCore.IGNORE);
		// Accs static 
		javaProject.setOption(JavaCore.COMPILER_PB_INDIRECT_STATIC_ACCESS, JavaCore.IGNORE);
		// Gestion des templates (ex List doit tre List<E>)
		javaProject.setOption(JavaCore.COMPILER_PB_RAW_TYPE_REFERENCE, JavaCore.IGNORE);
	}

	/**
	 * 
	 * @param javaProject
	 *            le projet eclipse
	 * @param libsToAdd
	 *            les librairies  ajouter au classpath
	 * @param exported
	 *            true si les librairies doivent tre exportes
	 * @throws JavaModelException
	 *             si erreur
	 */
	private void createAndAddLibsToClasspath(IJavaProject javaProject,
			Collection<String> libsToAdd, boolean exported)
			throws JavaModelException {
		String curLib;
		IClasspathEntry newLib;
		for (Iterator<String> it = libsToAdd.iterator(); it.hasNext();) {
			curLib = it.next();
			List<IClasspathEntry> cpEntriesList = new ArrayList<IClasspathEntry>(
					Arrays.asList(javaProject.getRawClasspath()));
			// On ajoute la librarie seulement si elle n'existe pas dj
			// sinon on aura une erreur
			newLib = JavaCore.newLibraryEntry(new Path(curLib), null, null,
					new IAccessRule[0], new IClasspathAttribute[0], exported);
			if (!cpEntriesList.contains(newLib)) {
				cpEntriesList.add(newLib);
				javaProject.setRawClasspath(cpEntriesList
						.toArray(new IClasspathEntry[cpEntriesList.size()]),
						true, null);
			}
			addToClasspath(curLib);
		}
	}

	/**
	 * Gestion du classpath. 1. Parcours les entres du .classpath et les ajoute
	 * au classpath de retour. 2. Filtre les entres connues 3. Supprime les
	 * entres qui pourront engendrer une erreur lors de la compilation car non
	 * rsolue.
	 * 
	 * @param cpEntries
	 *            les entres du .classpath  vrifier et  ajouter
	 * @param javaProject
	 *            le projet java
	 * @throws Exception
	 *             si erreur
	 */
	private void addToClasspath(IClasspathEntry[] cpEntries,
			IJavaProject javaProject) throws Exception {
		IClasspathContainer cpContainer;
		IPath entryPath;
		for (int i = 0; i < cpEntries.length; i++) {
			entryPath = cpEntries[i].getPath();
			if (IClasspathEntry.CPE_SOURCE == cpEntries[i].getEntryKind()) {
				// On ajoute les rpertoires  exclure de la compilation
				addExclusionPatterns(javaProject, cpEntries[i]);
				// Et on n'ajoute pas l'entre au classpath de retour
			} else if (IClasspathEntry.CPE_CONTAINER == cpEntries[i]
					.getEntryKind()) {
				// On ignore les JREContainer
				if (BuildWorkspaceConstants.JRE_CONTAINER.equals(entryPath
						.segment(0))) {
					// On supprime juste l'entre
					deleteEntry(cpEntries[i], javaProject);
				} else {
					try {
						// Rcupration spcifique car entre de type container
						cpContainer = JavaCore.getClasspathContainer(entryPath,
								javaProject);
						if (null != cpContainer) {
							// Rsolution du container, on ajoute les entres
							// associes
							addToClasspath(cpContainer.getClasspathEntries(),
									javaProject);
						} else {
							// On supprime l'entre car non rsolue
							deleteEntryWithWarning(cpEntries[i], javaProject);
						}
					} catch (JavaModelException e) {
						// On supprime l'entre car non rsolue
						deleteEntryWithWarning(cpEntries[i], javaProject);
					}
				}
			} else {
				// On ajoute les entres rsolues au classpath de retour
				addToClasspath(javaProject, cpEntries[i]);
			}
		}

	}

	/**
	 * Ajoute les patterns d'exclusion  l'entre de type source
	 * 
	 * @param javaProject
	 *            le projet java
	 * @param srcEntry
	 *            l'entre de type SOURCE
	 * @throws Exception
	 *             si erreur
	 */
	private void addExclusionPatterns(IJavaProject javaProject,
			IClasspathEntry srcEntry) throws Exception {
		// On cre une entre de type source en ajoutant les rpertoires exclus
		List<IPath> excludedPaths = new ArrayList<IPath>(Arrays.asList(srcEntry.getExclusionPatterns()));
		// On ajoute  ceux dj prsents les patterns donns en arguments de la
		// commande
		// On vrifie que le rpertoire fait parti des sources
		File src = new File(
				javaProject.getProject().getLocation().toOSString(),
				srcEntry.getPath().toOSString().replaceFirst("[/\\\\]" + javaProject.getProject().getName(), ""));
		for(int i=0; i<exclusionPatterns.length; i++) {
			if(exclusionPatterns[i].toOSString().startsWith(src.getAbsolutePath())) {
				// On l'ajoute aux exclusions
				String exPattern = exclusionPatterns[i].toOSString().replaceFirst(src.getAbsolutePath().replace("\\", "\\\\"), "") + "/**";
				exPattern = exPattern.replace('\\', '/');
				if(exPattern.startsWith("/")) {
					exPattern = exPattern.substring(1);
				}
				excludedPaths.add(new Path(exPattern));
			}
		}
		IClasspathEntry newSrcEntry = JavaCore.newSourceEntry(srcEntry
				.getPath(), excludedPaths.toArray(new IPath[excludedPaths
				.size()]));
		List<IClasspathEntry> cpEntriesList = new ArrayList<IClasspathEntry>(
				Arrays.asList(javaProject.getRawClasspath()));
		// On remplace l'entre donne en paramtre par la nouvelle entre cree
		cpEntriesList.set(cpEntriesList.indexOf(srcEntry), newSrcEntry);
		// On modifie le classpath du projet
		javaProject
				.setRawClasspath(cpEntriesList
						.toArray(new IClasspathEntry[cpEntriesList.size()]),
						true, null);
	}

	/**
	 * Essaye de rsoudre une entre grce aux arguments passs lors de l'appel
	 * et l'ajoute ou la supprime du .classpath selon le cas
	 * 
	 * PRECONDITION : Il ne faut pas que le classpath soit en readonly
	 * 
	 * @param entryToDelete
	 *            l'entre  supprimer
	 * @param javaProject
	 *            le projet java
	 * @throws Exception
	 *             si erreur
	 */
	private void deleteEntryWithWarning(IClasspathEntry entryToDelete,
			IJavaProject javaProject) throws Exception {
		// Avant de supprimer on regarde si l'entre ne correspond pas  une
		// entre
		// utilisateur (variable ou librairie)
		// On recherche donc dans les arguments si une variable de ce nom a
		// t dfinie
		// Dans le cas d'une librarie utilisateur, on supprime le premier
		// segment de l'entre
		// (JavaCore.USER_LIBRARY_CONTAINER_ID) pour ne rcuprer que le nom
		// donn par l'utilisateur
		/*
		 * String entryName = entryToDelete.getPath().toOSString().replaceFirst(
		 * JavaCore.USER_LIBRARY_CONTAINER_ID + "[/\\\\]?", ""); List<String>
		 * libs = varLibs.get(entryName); if (null != libs) { // On ajoute les
		 * libraries associes au classpath
		 * createAndAddLibsToClasspath(javaProject, libs, true); } else {
		 */
		// On log un warning
		String warningMsg = BuilderMessages.getString(
				"exception.unresolved_classpath_entry", new String[] {
						javaProject.getProject().getName(),
						entryToDelete.getPath().toOSString() });
		LOGGER.warn(warningMsg);
		// }
		// On supprime l'entre dans tous les cas
		deleteEntry(entryToDelete, javaProject);
	}

	/**
	 * Supprime une entre non rsolue de .classpath PRECONDITION : Il ne faut
	 * pas que le classpath soit en readonly
	 * 
	 * @param entryToDelete
	 *            l'entre  supprimer
	 * @param javaProject
	 *            le projet java
	 * @throws Exception
	 *             si erreur
	 */
	private void deleteEntry(IClasspathEntry entryToDelete,
			IJavaProject javaProject) throws Exception {
		// On supprime l'entre du classpath pour pouvoir compiler
		List<IClasspathEntry> cpEntriesList = new ArrayList<IClasspathEntry>(
				Arrays.asList(javaProject.getRawClasspath()));
		cpEntriesList.remove(entryToDelete);
		javaProject
				.setRawClasspath(cpEntriesList
						.toArray(new IClasspathEntry[cpEntriesList.size()]),
						true, null);
	}

	/**
	 * Ajoute une entre au classpath de retourne ou la supprime si elle ne peut
	 * pas tre rsolue.
	 * 
	 * PRECONDITION : Il ne faut pas que le classpath soit en readonly
	 * 
	 * @param javaProject
	 *            le projet java
	 * @param classpathEntry
	 *            l'entre du .classpath
	 * @throws Exception
	 *             si erreur
	 */
	private void addToClasspath(IJavaProject javaProject,
			IClasspathEntry classpathEntry) throws Exception {
		IClasspathEntry resolvedCpEntrie = JavaCore
				.getResolvedClasspathEntry(classpathEntry);
		if (null == resolvedCpEntrie) {
			// On supprime l'entre
			deleteEntryWithWarning(classpathEntry, javaProject);
		} else {
			// On l'ajoute au classpath
			File entryFile = resolvedCpEntrie.getPath().toFile();
			if (entryFile.exists() && entryFile.isAbsolute()) {
				addToClasspath(entryFile.getAbsolutePath());
			} else {
				// On essaye de rechercher une ressource
				IResource resource = wspace.getRoot().findMember(
						resolvedCpEntrie.getPath().toOSString());
				if (null != resource) {
					addToClasspath(resource.getLocation().toOSString());
				}
			}
		}
	}

	/**
	 * Ajoute un chemin au classpath de retour
	 * 
	 * @param pathToAdd
	 *            le chemin  ajouter
	 */
	private void addToClasspath(String pathToAdd) {
		if (!isOnClasspath(pathToAdd)) {
			classpath.append(convertToUnixPath(pathToAdd));
			classpath.append(";");
		}
	}

	/**
	 * Vrifie que le chemin n'est pas dj prsent dans le classpath de retour
	 * 
	 * @param pathToVerify
	 *            le chemin  vrifier
	 * @return true si prsent
	 */
	private boolean isOnClasspath(String pathToVerify) {
		return classpath.toString().matches(
				".*" + convertToUnixPath(pathToVerify) + ".*");
	}

	/**
	 * Converti un chemin de fichier au systme Unix pour avoir des comparaisons
	 * homognes
	 * 
	 * @param path
	 *            le chemin  convertir
	 * @return le chemin transform
	 */
	private String convertToUnixPath(String path) {
		return path.replace('\\', '/');
	}

	/**
	 * Modifie le dialect des sources
	 * 
	 * @param args
	 *            les arguments de la commande
	 * @param i
	 *            l'index de recherche
	 * @throws BuilderException
	 *             si erreur
	 */
	private void setDialect(String[] args, int i) throws BuilderException {
		if (i >= args.length) {
			throw new BuilderException("exception.options.dialect");
		}
		dialect = args[i];
	}

	/**
	 * Modifie le chemin vers la jre
	 * 
	 * @param args
	 *            les arguments de la commande
	 * @param i
	 *            l'index de recherche
	 * @return
	 * @throws BuilderException
	 *             si erreur
	 */
	private List<String> getOptionPaths(String args) throws BuilderException {
		return getOptionPaths(args, BuildWorkspaceConstants.OPTION_LIST_SEP);
	}

	/**
	 * Vrifie les chemins de fichiers passs en arguments de commande sous la
	 * forme path1<code>sep</code>path2
	 * 
	 * @param args
	 *            les arguments de la commande
	 * @param sep
	 *            le sparateur
	 * @return les chemins spars par <code>sep</code>
	 * @throws BuilderException
	 *             si erreur
	 */
	private List<String> getOptionPaths(String args, String sep)
			throws BuilderException {
		String[] paths = args.split(sep);
		File currentPath;
		// On vrifie tous les chemins donns
		for (int i = 0; i < paths.length; i++) {
			currentPath = new File(paths[i]);
			if (!currentPath.exists()) {
				// Le chamin indiqu n'existe pas, c'est une erreur de
				// configuration
				throw new BuilderException(BuilderMessages.getString(
						"exception.path.not_found", new String[] { paths[i] }));
			}
		}
		// On retourne la collection
		return Arrays.asList(paths);
	}
}